from __future__ import annotations

import numpy as np

import pytest

from mckit.parser.meshtal_parser import meshtal_lexer, meshtal_parser
from mckit.utils import path_resolver

file_resolver = path_resolver("tests")


@pytest.mark.parametrize(
    "mesh_file, expected",
    [
        (
            "parser_test_data/fmesh.m",
            {
                "date": "05/24/1812:21:45",
                "histories": 10000000.00,
                "title": "fmesh test",
                "tallies": [
                    {
                        "name": 14,
                        "particle": "NEUTRON",
                        "geom": "XYZ",
                        "bins": {
                            "ENERGY": np.array([0.00e00, 1.00e36]),
                            "X": np.array([-3.00, -1.00, 1.00, 3.00]),
                            "Y": np.array([-3.00, -1.00, 1.00, 3.00, 5.00]),
                            "Z": np.array([-4.00, -1.60, 0.80, 3.20, 5.60, 8.00]),
                        },
                        "result": np.array(
                            [
                                [
                                    [
                                        [
                                            7.18813e-03,
                                            3.06754e-03,
                                            1.46764e-03,
                                            7.66660e-04,
                                            4.03778e-04,
                                        ],
                                        [
                                            1.06634e-02,
                                            3.67344e-03,
                                            1.64976e-03,
                                            8.36825e-04,
                                            4.28819e-04,
                                        ],
                                        [
                                            7.17257e-03,
                                            3.06150e-03,
                                            1.47022e-03,
                                            7.71489e-04,
                                            3.98268e-04,
                                        ],
                                        [
                                            3.47599e-03,
                                            1.93182e-03,
                                            1.04730e-03,
                                            5.73177e-04,
                                            3.11514e-04,
                                        ],
                                    ],
                                    [
                                        [
                                            1.06743e-02,
                                            3.67618e-03,
                                            1.64985e-03,
                                            8.36179e-04,
                                            4.36976e-04,
                                        ],
                                        [
                                            2.01953e-02,
                                            4.44305e-03,
                                            1.83404e-03,
                                            9.03003e-04,
                                            4.60324e-04,
                                        ],
                                        [
                                            1.06720e-02,
                                            3.66319e-03,
                                            1.63652e-03,
                                            8.30197e-04,
                                            4.30752e-04,
                                        ],
                                        [
                                            4.24826e-03,
                                            2.24115e-03,
                                            1.18622e-03,
                                            6.40838e-04,
                                            3.43042e-04,
                                        ],
                                    ],
                                    [
                                        [
                                            7.21022e-03,
                                            3.08980e-03,
                                            1.48035e-03,
                                            7.68615e-04,
                                            4.02710e-04,
                                        ],
                                        [
                                            1.06646e-02,
                                            3.66923e-03,
                                            1.63603e-03,
                                            8.31973e-04,
                                            4.26487e-04,
                                        ],
                                        [
                                            7.19676e-03,
                                            3.07813e-03,
                                            1.47782e-03,
                                            7.74412e-04,
                                            4.05770e-04,
                                        ],
                                        [
                                            3.47633e-03,
                                            1.93512e-03,
                                            1.05235e-03,
                                            5.78607e-04,
                                            3.11651e-04,
                                        ],
                                    ],
                                ]
                            ]
                        ),
                        "error": np.array(
                            [
                                [
                                    [
                                        [0.00169, 0.00264, 0.00386, 0.00532, 0.00722],
                                        [0.00137, 0.00243, 0.00366, 0.00515, 0.00705],
                                        [0.00169, 0.00264, 0.00384, 0.00532, 0.00727],
                                        [0.00233, 0.00319, 0.00437, 0.00593, 0.00800],
                                    ],
                                    [
                                        [0.00137, 0.00243, 0.00367, 0.00514, 0.00698],
                                        [0.00093, 0.00221, 0.00350, 0.00497, 0.00682],
                                        [0.00137, 0.00243, 0.00368, 0.00517, 0.00703],
                                        [0.00212, 0.00302, 0.00419, 0.00572, 0.00774],
                                    ],
                                    [
                                        [0.00169, 0.00263, 0.00383, 0.00534, 0.00725],
                                        [0.00137, 0.00243, 0.00368, 0.00516, 0.00705],
                                        [0.00169, 0.00264, 0.00384, 0.00531, 0.00720],
                                        [0.00233, 0.00319, 0.00436, 0.00589, 0.00800],
                                    ],
                                ]
                            ]
                        ),
                    },
                    {
                        "name": 24,
                        "particle": "PHOTON",
                        "geom": "CYL",
                        "origin": np.array([-3.00e00, -3.00e00, -4.00e00]),
                        "axis": np.array([0.000e00, 0.000e00, 1.000e00]),
                        "bins": {
                            "ENERGY": np.array([0.00e00, 6.67e00, 1.33e01, 2.00e01]),
                            "R": np.array([0.00, 1.00, 2.00, 3.00]),
                            "Z": np.array([0.00, 1.25, 2.50, 3.75, 5.00]),
                            "THETA": np.array([0.000, 0.200, 0.400, 0.600, 0.800, 1.000]),
                        },
                        "result": np.array(
                            [
                                [
                                    [
                                        [
                                            4.26676e-04,
                                            3.60929e-04,
                                            2.51881e-04,
                                            2.35336e-04,
                                            3.27815e-04,
                                        ],
                                        [
                                            4.17376e-04,
                                            3.74891e-04,
                                            2.65612e-04,
                                            2.47303e-04,
                                            3.41729e-04,
                                        ],
                                        [
                                            3.52530e-04,
                                            3.21725e-04,
                                            2.39415e-04,
                                            2.22406e-04,
                                            2.87633e-04,
                                        ],
                                        [
                                            2.91647e-04,
                                            2.64957e-04,
                                            2.03075e-04,
                                            1.88207e-04,
                                            2.53038e-04,
                                        ],
                                    ],
                                    [
                                        [
                                            5.72802e-04,
                                            4.02577e-04,
                                            1.62808e-04,
                                            1.47284e-04,
                                            3.22879e-04,
                                        ],
                                        [
                                            5.32281e-04,
                                            3.90955e-04,
                                            1.77012e-04,
                                            1.56642e-04,
                                            3.20700e-04,
                                        ],
                                        [
                                            4.34871e-04,
                                            3.36510e-04,
                                            1.64430e-04,
                                            1.47865e-04,
                                            2.81179e-04,
                                        ],
                                        [
                                            3.39817e-04,
                                            2.74729e-04,
                                            1.44850e-04,
                                            1.30022e-04,
                                            2.32783e-04,
                                        ],
                                    ],
                                    [
                                        [
                                            7.68761e-04,
                                            4.02569e-04,
                                            1.09868e-04,
                                            9.83327e-05,
                                            2.85524e-04,
                                        ],
                                        [
                                            6.74185e-04,
                                            3.99908e-04,
                                            1.23025e-04,
                                            1.08080e-04,
                                            2.97412e-04,
                                        ],
                                        [
                                            5.11027e-04,
                                            3.42010e-04,
                                            1.15448e-04,
                                            1.05403e-04,
                                            2.57364e-04,
                                        ],
                                        [
                                            3.89689e-04,
                                            2.67553e-04,
                                            1.05158e-04,
                                            9.46154e-05,
                                            2.10578e-04,
                                        ],
                                    ],
                                ],
                                [
                                    [
                                        [
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            1.24150e-07,
                                            5.58994e-09,
                                            1.24827e-07,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            5.57853e-09,
                                            2.64984e-08,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                    ],
                                    [
                                        [
                                            3.31041e-08,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            4.24567e-08,
                                            4.34742e-08,
                                            4.24567e-08,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            0.00000e00,
                                            1.51344e-08,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            9.82692e-09,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                    ],
                                    [
                                        [
                                            3.44266e-08,
                                            3.21212e-08,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            4.55038e-08,
                                            3.04180e-08,
                                            2.54702e-08,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            3.03939e-09,
                                            3.13872e-08,
                                            4.05251e-09,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            2.19894e-09,
                                            4.43644e-09,
                                            3.78085e-08,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                    ],
                                ],
                                [
                                    [
                                        [
                                            1.61383e-07,
                                            2.45465e-07,
                                            4.80361e-08,
                                            2.86873e-08,
                                            1.36531e-07,
                                        ],
                                        [
                                            0.00000e00,
                                            0.00000e00,
                                            6.11344e-08,
                                            6.62558e-08,
                                            6.94414e-08,
                                        ],
                                        [
                                            1.79183e-07,
                                            1.83093e-07,
                                            1.26324e-07,
                                            1.10180e-07,
                                            9.84652e-08,
                                        ],
                                        [
                                            1.15801e-07,
                                            7.11787e-08,
                                            9.61169e-08,
                                            6.27373e-08,
                                            9.09577e-08,
                                        ],
                                    ],
                                    [
                                        [
                                            3.33988e-07,
                                            3.02723e-07,
                                            1.00276e-07,
                                            4.54345e-08,
                                            1.28529e-07,
                                        ],
                                        [
                                            1.05911e-07,
                                            0.00000e00,
                                            6.03481e-08,
                                            6.20807e-08,
                                            6.65379e-08,
                                        ],
                                        [
                                            1.03901e-07,
                                            1.16357e-07,
                                            1.77282e-07,
                                            6.12241e-08,
                                            1.68246e-07,
                                        ],
                                        [
                                            6.42099e-08,
                                            5.82408e-08,
                                            3.50897e-08,
                                            1.41587e-07,
                                            6.02894e-08,
                                        ],
                                    ],
                                    [
                                        [
                                            4.35790e-07,
                                            5.28413e-08,
                                            6.09709e-08,
                                            2.61146e-08,
                                            2.55932e-07,
                                        ],
                                        [
                                            1.36592e-07,
                                            2.03173e-07,
                                            6.40422e-08,
                                            2.55531e-08,
                                            1.59063e-07,
                                        ],
                                        [
                                            2.33628e-07,
                                            3.94338e-07,
                                            1.14898e-07,
                                            1.76035e-08,
                                            9.71856e-08,
                                        ],
                                        [
                                            2.10624e-07,
                                            1.56979e-07,
                                            7.03514e-08,
                                            4.10227e-08,
                                            1.43260e-07,
                                        ],
                                    ],
                                ],
                            ]
                        ),
                        "error": np.array(
                            [
                                [
                                    [
                                        [
                                            1.51165e-02,
                                            1.65362e-02,
                                            1.95638e-02,
                                            2.00471e-02,
                                            1.74294e-02,
                                        ],
                                        [
                                            1.54432e-02,
                                            1.65796e-02,
                                            1.93011e-02,
                                            1.98625e-02,
                                            1.73169e-02,
                                        ],
                                        [
                                            1.70852e-02,
                                            1.80477e-02,
                                            2.04493e-02,
                                            2.12808e-02,
                                            1.90279e-02,
                                        ],
                                        [
                                            1.91640e-02,
                                            2.00755e-02,
                                            2.24192e-02,
                                            2.31940e-02,
                                            2.06925e-02,
                                        ],
                                    ],
                                    [
                                        [
                                            9.09560e-03,
                                            1.13354e-02,
                                            1.68719e-02,
                                            1.73306e-02,
                                            1.27127e-02,
                                        ],
                                        [
                                            9.56337e-03,
                                            1.15957e-02,
                                            1.62495e-02,
                                            1.68587e-02,
                                            1.28723e-02,
                                        ],
                                        [
                                            1.07933e-02,
                                            1.25307e-02,
                                            1.69082e-02,
                                            1.73571e-02,
                                            1.37630e-02,
                                        ],
                                        [
                                            1.21292e-02,
                                            1.38951e-02,
                                            1.80609e-02,
                                            1.84843e-02,
                                            1.50679e-02,
                                        ],
                                    ],
                                    [
                                        [
                                            6.46985e-03,
                                            9.39907e-03,
                                            1.63465e-02,
                                            1.65062e-02,
                                            1.11913e-02,
                                        ],
                                        [
                                            7.10451e-03,
                                            9.57794e-03,
                                            1.55160e-02,
                                            1.59039e-02,
                                            1.11509e-02,
                                        ],
                                        [
                                            8.19050e-03,
                                            1.02979e-02,
                                            1.59845e-02,
                                            1.59556e-02,
                                            1.17764e-02,
                                        ],
                                        [
                                            9.40942e-03,
                                            1.15396e-02,
                                            1.66056e-02,
                                            1.68655e-02,
                                            1.28912e-02,
                                        ],
                                    ],
                                ],
                                [
                                    [
                                        [
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            1.00000e00,
                                            1.00000e00,
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            1.00000e00,
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                    ],
                                    [
                                        [
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            1.00000e00,
                                            1.00000e00,
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            0.00000e00,
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                    ],
                                    [
                                        [
                                            1.00000e00,
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            7.94765e-01,
                                            1.00000e00,
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            1.00000e00,
                                            1.00000e00,
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            1.00000e00,
                                            1.00000e00,
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                    ],
                                ],
                                [
                                    [
                                        [
                                            7.13649e-01,
                                            7.08199e-01,
                                            7.92007e-01,
                                            1.00000e00,
                                            7.98367e-01,
                                        ],
                                        [
                                            0.00000e00,
                                            0.00000e00,
                                            1.00000e00,
                                            1.00000e00,
                                            1.00000e00,
                                        ],
                                        [
                                            7.37976e-01,
                                            7.34192e-01,
                                            8.08038e-01,
                                            1.00000e00,
                                            9.90367e-01,
                                        ],
                                        [
                                            1.00000e00,
                                            1.00000e00,
                                            1.00000e00,
                                            7.11428e-01,
                                            1.00000e00,
                                        ],
                                    ],
                                    [
                                        [
                                            3.73939e-01,
                                            4.35674e-01,
                                            6.39167e-01,
                                            1.00000e00,
                                            6.13512e-01,
                                        ],
                                        [
                                            7.09827e-01,
                                            0.00000e00,
                                            8.65521e-01,
                                            8.21311e-01,
                                            7.07324e-01,
                                        ],
                                        [
                                            7.06117e-01,
                                            6.58996e-01,
                                            5.87822e-01,
                                            8.92465e-01,
                                            5.85102e-01,
                                        ],
                                        [
                                            5.99586e-01,
                                            7.93568e-01,
                                            1.00000e00,
                                            5.39828e-01,
                                            1.00000e00,
                                        ],
                                    ],
                                    [
                                        [
                                            2.74749e-01,
                                            6.07463e-01,
                                            7.11666e-01,
                                            1.00000e00,
                                            3.38896e-01,
                                        ],
                                        [
                                            4.70422e-01,
                                            4.07795e-01,
                                            7.12197e-01,
                                            1.00000e00,
                                            4.89133e-01,
                                        ],
                                        [
                                            4.69333e-01,
                                            3.23265e-01,
                                            4.92014e-01,
                                            8.57334e-01,
                                            5.38949e-01,
                                        ],
                                        [
                                            3.71867e-01,
                                            4.36175e-01,
                                            7.21109e-01,
                                            7.29208e-01,
                                            4.55335e-01,
                                        ],
                                    ],
                                ],
                            ]
                        ),
                    },
                    {
                        "name": 34,
                        "particle": "ELECTRON",
                        "geom": "XYZ",
                        "bins": {
                            "ENERGY": np.array([0.00e00, 6.67e00, 1.33e01, 2.00e01]),
                            "X": np.array([-3.00, -1.00, 1.00, 3.00]),
                            "Y": np.array([-3.00, -1.00, 1.00, 3.00, 5.00]),
                            "Z": np.array([-4.00, -1.60, 0.80, 3.20, 5.60, 8.00]),
                        },
                        "result": np.array(
                            [
                                [
                                    [
                                        [
                                            2.15470e-05,
                                            1.47842e-05,
                                            8.41469e-06,
                                            5.31042e-06,
                                            2.58881e-06,
                                        ],
                                        [
                                            2.90692e-05,
                                            1.77388e-05,
                                            9.66903e-06,
                                            5.45955e-06,
                                            3.11251e-06,
                                        ],
                                        [
                                            2.20583e-05,
                                            1.50941e-05,
                                            8.60972e-06,
                                            5.05476e-06,
                                            3.02462e-06,
                                        ],
                                        [
                                            1.09006e-05,
                                            9.11214e-06,
                                            6.48347e-06,
                                            3.71926e-06,
                                            2.25027e-06,
                                        ],
                                    ],
                                    [
                                        [
                                            2.93677e-05,
                                            1.70248e-05,
                                            1.00628e-05,
                                            5.66025e-06,
                                            3.11381e-06,
                                        ],
                                        [
                                            4.13888e-05,
                                            2.01021e-05,
                                            1.08080e-05,
                                            6.11828e-06,
                                            3.37041e-06,
                                        ],
                                        [
                                            2.85739e-05,
                                            1.71446e-05,
                                            9.64213e-06,
                                            5.33722e-06,
                                            3.04467e-06,
                                        ],
                                        [
                                            1.38698e-05,
                                            1.14807e-05,
                                            6.74720e-06,
                                            4.62259e-06,
                                            2.49310e-06,
                                        ],
                                    ],
                                    [
                                        [
                                            2.14460e-05,
                                            1.51609e-05,
                                            8.42577e-06,
                                            5.04945e-06,
                                            2.89669e-06,
                                        ],
                                        [
                                            2.77666e-05,
                                            1.71581e-05,
                                            1.01360e-05,
                                            5.69297e-06,
                                            3.08384e-06,
                                        ],
                                        [
                                            2.15495e-05,
                                            1.43817e-05,
                                            8.53499e-06,
                                            5.07401e-06,
                                            2.83177e-06,
                                        ],
                                        [
                                            1.13652e-05,
                                            8.80429e-06,
                                            5.28008e-06,
                                            3.52305e-06,
                                            2.07175e-06,
                                        ],
                                    ],
                                ],
                                [
                                    [
                                        [
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            3.26852e-09,
                                            9.89135e-09,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            7.96285e-09,
                                            9.83845e-09,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            1.13799e-08,
                                            0.00000e00,
                                        ],
                                    ],
                                    [
                                        [
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            1.52051e-08,
                                            1.18793e-08,
                                            3.44674e-09,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            1.26318e-08,
                                            0.00000e00,
                                            0.00000e00,
                                            6.67205e-09,
                                            0.00000e00,
                                        ],
                                        [
                                            9.56817e-09,
                                            3.09187e-09,
                                            7.86285e-09,
                                            1.18994e-09,
                                            0.00000e00,
                                        ],
                                    ],
                                    [
                                        [
                                            4.66316e-09,
                                            7.48677e-09,
                                            2.88179e-09,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            0.00000e00,
                                            1.37866e-08,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            8.21988e-09,
                                            2.23643e-08,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            1.70057e-09,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                    ],
                                ],
                                [
                                    [
                                        [
                                            3.99267e-09,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            1.43729e-09,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            3.48114e-09,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                    ],
                                    [
                                        [
                                            1.09841e-08,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            0.00000e00,
                                            0.00000e00,
                                            1.66217e-09,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            4.72261e-09,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            0.00000e00,
                                            8.79939e-10,
                                            9.98414e-09,
                                            8.32928e-09,
                                            0.00000e00,
                                        ],
                                    ],
                                    [
                                        [
                                            1.09076e-08,
                                            2.46367e-09,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            8.07081e-09,
                                            4.18543e-09,
                                            2.82902e-09,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                    ],
                                ],
                            ]
                        ),
                        "error": np.array(
                            [
                                [
                                    [
                                        [0.01902, 0.02288, 0.03020, 0.03851, 0.05383],
                                        [0.01640, 0.02090, 0.02820, 0.03750, 0.04963],
                                        [0.01886, 0.02272, 0.03020, 0.03915, 0.05082],
                                        [0.02834, 0.03076, 0.03680, 0.04882, 0.06265],
                                    ],
                                    [
                                        [0.01637, 0.02122, 0.02782, 0.03681, 0.04955],
                                        [0.01390, 0.01960, 0.02668, 0.03578, 0.04802],
                                        [0.01662, 0.02122, 0.02836, 0.03747, 0.04925],
                                        [0.02380, 0.02623, 0.03377, 0.04113, 0.05499],
                                    ],
                                    [
                                        [0.01907, 0.02275, 0.03038, 0.03881, 0.05075],
                                        [0.01681, 0.02130, 0.02777, 0.03607, 0.04907],
                                        [0.01913, 0.02315, 0.03017, 0.03899, 0.05175],
                                        [0.02756, 0.03115, 0.03975, 0.04945, 0.06481],
                                    ],
                                ],
                                [
                                    [
                                        [0.00000, 0.00000, 0.00000, 0.00000, 0.00000],
                                        [1.00000, 1.00000, 0.00000, 0.00000, 0.00000],
                                        [1.00000, 1.00000, 0.00000, 0.00000, 0.00000],
                                        [0.00000, 0.00000, 0.00000, 1.00000, 0.00000],
                                    ],
                                    [
                                        [0.00000, 0.00000, 0.00000, 0.00000, 0.00000],
                                        [0.82592, 1.00000, 1.00000, 0.00000, 0.00000],
                                        [0.70981, 0.00000, 0.00000, 1.00000, 0.00000],
                                        [0.93217, 1.00000, 1.00000, 1.00000, 0.00000],
                                    ],
                                    [
                                        [1.00000, 0.77600, 1.00000, 0.00000, 0.00000],
                                        [0.00000, 0.89989, 0.00000, 0.00000, 0.00000],
                                        [0.77883, 0.76382, 0.00000, 0.00000, 0.00000],
                                        [1.00000, 0.00000, 0.00000, 0.00000, 0.00000],
                                    ],
                                ],
                                [
                                    [
                                        [1.00000, 0.00000, 0.00000, 0.00000, 0.00000],
                                        [1.00000, 0.00000, 0.00000, 0.00000, 0.00000],
                                        [1.00000, 0.00000, 0.00000, 0.00000, 0.00000],
                                        [0.00000, 0.00000, 0.00000, 0.00000, 0.00000],
                                    ],
                                    [
                                        [1.00000, 0.00000, 0.00000, 0.00000, 0.00000],
                                        [0.00000, 0.00000, 1.00000, 0.00000, 0.00000],
                                        [1.00000, 0.00000, 0.00000, 0.00000, 0.00000],
                                        [0.00000, 1.00000, 1.00000, 1.00000, 0.00000],
                                    ],
                                    [
                                        [1.00000, 0.74901, 0.00000, 0.00000, 0.00000],
                                        [0.00000, 0.00000, 0.00000, 0.00000, 0.00000],
                                        [1.00000, 1.00000, 1.00000, 0.00000, 0.00000],
                                        [0.00000, 0.00000, 0.00000, 0.00000, 0.00000],
                                    ],
                                ],
                            ]
                        ),
                    },
                    {
                        "name": 44,
                        "particle": "NEUTRON",
                        "geom": "CYL",
                        "origin": np.array([-3.00e00, -3.00e00, -4.00e00]),
                        "axis": np.array([0.000e00, 0.000e00, 1.000e00]),
                        "bins": {
                            "ENERGY": np.array([0.00e00, 6.67e00, 1.33e01, 2.00e01]),
                            "R": np.array([0.00, 1.00, 2.00, 3.00]),
                            "Z": np.array([0.00, 1.25, 2.50, 3.75, 5.00]),
                            "THETA": np.array([0.000, 0.200, 0.400, 0.600, 0.800, 1.000]),
                        },
                        "result": np.array(
                            [
                                [
                                    [
                                        [
                                            6.36513e-04,
                                            5.39688e-04,
                                            3.56988e-04,
                                            3.25666e-04,
                                            4.90815e-04,
                                        ],
                                        [
                                            6.81107e-04,
                                            5.81260e-04,
                                            3.95854e-04,
                                            3.69511e-04,
                                            5.38530e-04,
                                        ],
                                        [
                                            5.91488e-04,
                                            5.24466e-04,
                                            3.66878e-04,
                                            3.32395e-04,
                                            4.83348e-04,
                                        ],
                                        [
                                            4.84344e-04,
                                            4.19087e-04,
                                            2.96782e-04,
                                            2.80874e-04,
                                            3.97798e-04,
                                        ],
                                    ],
                                    [
                                        [
                                            8.75449e-04,
                                            5.96380e-04,
                                            2.18626e-04,
                                            1.89416e-04,
                                            4.71914e-04,
                                        ],
                                        [
                                            9.14165e-04,
                                            6.39521e-04,
                                            2.54077e-04,
                                            2.16996e-04,
                                            5.13943e-04,
                                        ],
                                        [
                                            7.60698e-04,
                                            5.53425e-04,
                                            2.37453e-04,
                                            2.02494e-04,
                                            4.51437e-04,
                                        ],
                                        [
                                            5.99848e-04,
                                            4.56087e-04,
                                            2.05150e-04,
                                            1.78299e-04,
                                            3.79469e-04,
                                        ],
                                    ],
                                    [
                                        [
                                            1.16547e-03,
                                            6.10956e-04,
                                            1.44214e-04,
                                            1.28095e-04,
                                            4.19222e-04,
                                        ],
                                        [
                                            1.12122e-03,
                                            6.39670e-04,
                                            1.66427e-04,
                                            1.46253e-04,
                                            4.61955e-04,
                                        ],
                                        [
                                            9.04913e-04,
                                            5.66029e-04,
                                            1.66111e-04,
                                            1.43689e-04,
                                            4.10153e-04,
                                        ],
                                        [
                                            6.83654e-04,
                                            4.49291e-04,
                                            1.49303e-04,
                                            1.32920e-04,
                                            3.40205e-04,
                                        ],
                                    ],
                                ],
                                [
                                    [
                                        [
                                            5.68879e-04,
                                            5.00413e-04,
                                            3.63351e-04,
                                            3.51731e-04,
                                            4.65246e-04,
                                        ],
                                        [
                                            5.74251e-04,
                                            5.13881e-04,
                                            3.69454e-04,
                                            3.48172e-04,
                                            4.67461e-04,
                                        ],
                                        [
                                            4.55839e-04,
                                            4.16947e-04,
                                            3.17566e-04,
                                            3.00836e-04,
                                            3.90086e-04,
                                        ],
                                        [
                                            3.52446e-04,
                                            3.26314e-04,
                                            2.55279e-04,
                                            2.45904e-04,
                                            3.07041e-04,
                                        ],
                                    ],
                                    [
                                        [
                                            7.44170e-04,
                                            5.41893e-04,
                                            2.34327e-04,
                                            2.04675e-04,
                                            4.49847e-04,
                                        ],
                                        [
                                            7.15024e-04,
                                            5.39231e-04,
                                            2.49338e-04,
                                            2.21067e-04,
                                            4.58030e-04,
                                        ],
                                        [
                                            5.52225e-04,
                                            4.30963e-04,
                                            2.22508e-04,
                                            1.99851e-04,
                                            3.75739e-04,
                                        ],
                                        [
                                            4.16330e-04,
                                            3.40119e-04,
                                            1.84454e-04,
                                            1.72092e-04,
                                            2.95224e-04,
                                        ],
                                    ],
                                    [
                                        [
                                            9.82620e-04,
                                            5.47755e-04,
                                            1.53812e-04,
                                            1.37189e-04,
                                            3.99798e-04,
                                        ],
                                        [
                                            8.61145e-04,
                                            5.31881e-04,
                                            1.65989e-04,
                                            1.45397e-04,
                                            4.07246e-04,
                                        ],
                                        [
                                            6.41086e-04,
                                            4.28584e-04,
                                            1.60945e-04,
                                            1.41904e-04,
                                            3.40225e-04,
                                        ],
                                        [
                                            4.62649e-04,
                                            3.29253e-04,
                                            1.41001e-04,
                                            1.28809e-04,
                                            2.68768e-04,
                                        ],
                                    ],
                                ],
                                [
                                    [
                                        [
                                            4.47611e-03,
                                            3.76194e-03,
                                            2.78037e-03,
                                            2.66749e-03,
                                            3.44401e-03,
                                        ],
                                        [
                                            2.80391e-03,
                                            2.42859e-03,
                                            1.86302e-03,
                                            1.80358e-03,
                                            2.24376e-03,
                                        ],
                                        [
                                            1.77640e-03,
                                            1.60347e-03,
                                            1.29141e-03,
                                            1.27639e-03,
                                            1.52562e-03,
                                        ],
                                        [
                                            1.14696e-03,
                                            1.05410e-03,
                                            9.04880e-04,
                                            8.62988e-04,
                                            1.00358e-03,
                                        ],
                                    ],
                                    [
                                        [
                                            6.74332e-03,
                                            4.23841e-03,
                                            2.21247e-03,
                                            2.08460e-03,
                                            3.42436e-03,
                                        ],
                                        [
                                            3.83163e-03,
                                            2.68914e-03,
                                            1.53160e-03,
                                            1.43832e-03,
                                            2.25032e-03,
                                        ],
                                        [
                                            2.21608e-03,
                                            1.70459e-03,
                                            1.10556e-03,
                                            1.05776e-03,
                                            1.49573e-03,
                                        ],
                                        [
                                            1.35022e-03,
                                            1.09847e-03,
                                            8.04995e-04,
                                            7.82749e-04,
                                            9.96630e-04,
                                        ],
                                    ],
                                    [
                                        [
                                            1.09575e-02,
                                            4.60953e-03,
                                            1.78375e-03,
                                            1.65950e-03,
                                            3.25593e-03,
                                        ],
                                        [
                                            5.19313e-03,
                                            2.77331e-03,
                                            1.26611e-03,
                                            1.17277e-03,
                                            2.12967e-03,
                                        ],
                                        [
                                            2.70508e-03,
                                            1.73632e-03,
                                            9.59336e-04,
                                            9.03811e-04,
                                            1.44242e-03,
                                        ],
                                        [
                                            1.54029e-03,
                                            1.12071e-03,
                                            7.15744e-04,
                                            6.89978e-04,
                                            9.75051e-04,
                                        ],
                                    ],
                                ],
                            ]
                        ),
                        "error": np.array(
                            [
                                [
                                    [
                                        [0.01224, 0.01342, 0.01593, 0.01652, 0.01406],
                                        [0.01208, 0.01310, 0.01534, 0.01564, 0.01366],
                                        [0.01316, 0.01387, 0.01612, 0.01668, 0.01449],
                                        [0.01451, 0.01572, 0.01797, 0.01826, 0.01614],
                                    ],
                                    [
                                        [0.00731, 0.00910, 0.01382, 0.01457, 0.01018],
                                        [0.00743, 0.00900, 0.01303, 0.01360, 0.00988],
                                        [0.00817, 0.00959, 0.01342, 0.01411, 0.01052],
                                        [0.00927, 0.01064, 0.01440, 0.01499, 0.01148],
                                    ],
                                    [
                                        [0.00525, 0.00742, 0.01359, 0.01385, 0.00887],
                                        [0.00556, 0.00738, 0.01271, 0.01300, 0.00856],
                                        [0.00622, 0.00788, 0.01271, 0.01307, 0.00904],
                                        [0.00716, 0.00878, 0.01334, 0.01361, 0.00988],
                                    ],
                                ],
                                [
                                    [
                                        [0.01210, 0.01309, 0.01519, 0.01543, 0.01356],
                                        [0.01227, 0.01311, 0.01520, 0.01556, 0.01362],
                                        [0.01389, 0.01467, 0.01663, 0.01696, 0.01510],
                                        [0.01599, 0.01676, 0.01851, 0.01884, 0.01717],
                                    ],
                                    [
                                        [0.00732, 0.00902, 0.01334, 0.01399, 0.00996],
                                        [0.00760, 0.00911, 0.01294, 0.01350, 0.00990],
                                        [0.00867, 0.01007, 0.01367, 0.01414, 0.01088],
                                        [0.01000, 0.01134, 0.01499, 0.01520, 0.01226],
                                    ],
                                    [
                                        [0.00523, 0.00746, 0.01313, 0.01339, 0.00882],
                                        [0.00566, 0.00757, 0.01265, 0.01288, 0.00877],
                                        [0.00657, 0.00835, 0.01281, 0.01309, 0.00944],
                                        [0.00771, 0.00940, 0.01360, 0.01371, 0.01052],
                                    ],
                                ],
                                [
                                    [
                                        [0.00434, 0.00507, 0.00568, 0.00583, 0.00534],
                                        [0.00567, 0.00645, 0.00705, 0.00721, 0.00671],
                                        [0.00742, 0.00819, 0.00871, 0.00880, 0.00838],
                                        [0.00954, 0.01034, 0.01069, 0.01089, 0.01049],
                                    ],
                                    [
                                        [0.00238, 0.00382, 0.00439, 0.00438, 0.00422],
                                        [0.00326, 0.00458, 0.00527, 0.00528, 0.00502],
                                        [0.00437, 0.00555, 0.00624, 0.00621, 0.00595],
                                        [0.00565, 0.00677, 0.00738, 0.00733, 0.00711],
                                    ],
                                    [
                                        [0.00156, 0.00315, 0.00385, 0.00387, 0.00354],
                                        [0.00229, 0.00366, 0.00454, 0.00458, 0.00412],
                                        [0.00319, 0.00439, 0.00525, 0.00526, 0.00480],
                                        [0.00424, 0.00531, 0.00610, 0.00606, 0.00567],
                                    ],
                                ],
                            ]
                        ),
                    },
                    {
                        "name": 54,
                        "particle": "NEUTRON",
                        "geom": "CYL",
                        "origin": np.array([-3.00e00, -3.00e00, -4.00e00]),
                        "axis": np.array([0.000e00, 0.000e00, 1.000e00]),
                        "bins": {
                            "ENERGY": np.array([0.00e00, 6.67e00, 1.33e01, 2.00e01]),
                            "R": np.array([0.00, 1.00, 2.00, 3.00]),
                            "Z": np.array([0.00, 1.25, 2.50, 3.75, 5.00]),
                            "THETA": np.array([0.000, 0.200, 0.400, 0.600, 0.800, 1.000]),
                        },
                        "result": np.array(
                            [
                                [
                                    [
                                        [
                                            6.36513e-04,
                                            5.39688e-04,
                                            3.56988e-04,
                                            3.25666e-04,
                                            4.90815e-04,
                                        ],
                                        [
                                            6.81107e-04,
                                            5.81260e-04,
                                            3.95854e-04,
                                            3.69511e-04,
                                            5.38530e-04,
                                        ],
                                        [
                                            5.91488e-04,
                                            5.24466e-04,
                                            3.66878e-04,
                                            3.32395e-04,
                                            4.83348e-04,
                                        ],
                                        [
                                            4.84344e-04,
                                            4.19087e-04,
                                            2.96782e-04,
                                            2.80874e-04,
                                            3.97798e-04,
                                        ],
                                    ],
                                    [
                                        [
                                            8.75449e-04,
                                            5.96380e-04,
                                            2.18626e-04,
                                            1.89416e-04,
                                            4.71914e-04,
                                        ],
                                        [
                                            9.14165e-04,
                                            6.39521e-04,
                                            2.54077e-04,
                                            2.16996e-04,
                                            5.13943e-04,
                                        ],
                                        [
                                            7.60698e-04,
                                            5.53425e-04,
                                            2.37453e-04,
                                            2.02494e-04,
                                            4.51437e-04,
                                        ],
                                        [
                                            5.99848e-04,
                                            4.56087e-04,
                                            2.05150e-04,
                                            1.78299e-04,
                                            3.79469e-04,
                                        ],
                                    ],
                                    [
                                        [
                                            1.16547e-03,
                                            6.10956e-04,
                                            1.44214e-04,
                                            1.28095e-04,
                                            4.19222e-04,
                                        ],
                                        [
                                            1.12122e-03,
                                            6.39670e-04,
                                            1.66427e-04,
                                            1.46253e-04,
                                            4.61955e-04,
                                        ],
                                        [
                                            9.04913e-04,
                                            5.66029e-04,
                                            1.66111e-04,
                                            1.43689e-04,
                                            4.10153e-04,
                                        ],
                                        [
                                            6.83654e-04,
                                            4.49291e-04,
                                            1.49303e-04,
                                            1.32920e-04,
                                            3.40205e-04,
                                        ],
                                    ],
                                ],
                                [
                                    [
                                        [
                                            5.68879e-04,
                                            5.00413e-04,
                                            3.63351e-04,
                                            3.51731e-04,
                                            4.65246e-04,
                                        ],
                                        [
                                            5.74251e-04,
                                            5.13881e-04,
                                            3.69454e-04,
                                            3.48172e-04,
                                            4.67461e-04,
                                        ],
                                        [
                                            4.55839e-04,
                                            4.16947e-04,
                                            3.17566e-04,
                                            3.00836e-04,
                                            3.90086e-04,
                                        ],
                                        [
                                            3.52446e-04,
                                            3.26314e-04,
                                            2.55279e-04,
                                            2.45904e-04,
                                            3.07041e-04,
                                        ],
                                    ],
                                    [
                                        [
                                            7.44170e-04,
                                            5.41893e-04,
                                            2.34327e-04,
                                            2.04675e-04,
                                            4.49847e-04,
                                        ],
                                        [
                                            7.15024e-04,
                                            5.39231e-04,
                                            2.49338e-04,
                                            2.21067e-04,
                                            4.58030e-04,
                                        ],
                                        [
                                            5.52225e-04,
                                            4.30963e-04,
                                            2.22508e-04,
                                            1.99851e-04,
                                            3.75739e-04,
                                        ],
                                        [
                                            4.16330e-04,
                                            3.40119e-04,
                                            1.84454e-04,
                                            1.72092e-04,
                                            2.95224e-04,
                                        ],
                                    ],
                                    [
                                        [
                                            9.82620e-04,
                                            5.47755e-04,
                                            1.53812e-04,
                                            1.37189e-04,
                                            3.99798e-04,
                                        ],
                                        [
                                            8.61145e-04,
                                            5.31881e-04,
                                            1.65989e-04,
                                            1.45397e-04,
                                            4.07246e-04,
                                        ],
                                        [
                                            6.41086e-04,
                                            4.28584e-04,
                                            1.60945e-04,
                                            1.41904e-04,
                                            3.40225e-04,
                                        ],
                                        [
                                            4.62649e-04,
                                            3.29253e-04,
                                            1.41001e-04,
                                            1.28809e-04,
                                            2.68768e-04,
                                        ],
                                    ],
                                ],
                                [
                                    [
                                        [
                                            4.47611e-03,
                                            3.76194e-03,
                                            2.78037e-03,
                                            2.66749e-03,
                                            3.44401e-03,
                                        ],
                                        [
                                            2.80391e-03,
                                            2.42859e-03,
                                            1.86302e-03,
                                            1.80358e-03,
                                            2.24376e-03,
                                        ],
                                        [
                                            1.77640e-03,
                                            1.60347e-03,
                                            1.29141e-03,
                                            1.27639e-03,
                                            1.52562e-03,
                                        ],
                                        [
                                            1.14696e-03,
                                            1.05410e-03,
                                            9.04880e-04,
                                            8.62988e-04,
                                            1.00358e-03,
                                        ],
                                    ],
                                    [
                                        [
                                            6.74332e-03,
                                            4.23841e-03,
                                            2.21247e-03,
                                            2.08460e-03,
                                            3.42436e-03,
                                        ],
                                        [
                                            3.83163e-03,
                                            2.68914e-03,
                                            1.53160e-03,
                                            1.43832e-03,
                                            2.25032e-03,
                                        ],
                                        [
                                            2.21608e-03,
                                            1.70459e-03,
                                            1.10556e-03,
                                            1.05776e-03,
                                            1.49573e-03,
                                        ],
                                        [
                                            1.35022e-03,
                                            1.09847e-03,
                                            8.04995e-04,
                                            7.82749e-04,
                                            9.96630e-04,
                                        ],
                                    ],
                                    [
                                        [
                                            1.09575e-02,
                                            4.60953e-03,
                                            1.78375e-03,
                                            1.65950e-03,
                                            3.25593e-03,
                                        ],
                                        [
                                            5.19313e-03,
                                            2.77331e-03,
                                            1.26611e-03,
                                            1.17277e-03,
                                            2.12967e-03,
                                        ],
                                        [
                                            2.70508e-03,
                                            1.73632e-03,
                                            9.59336e-04,
                                            9.03811e-04,
                                            1.44242e-03,
                                        ],
                                        [
                                            1.54029e-03,
                                            1.12071e-03,
                                            7.15744e-04,
                                            6.89978e-04,
                                            9.75051e-04,
                                        ],
                                    ],
                                ],
                            ]
                        ),
                        "error": np.array(
                            [
                                [
                                    [
                                        [0.01224, 0.01342, 0.01593, 0.01652, 0.01406],
                                        [0.01208, 0.01310, 0.01534, 0.01564, 0.01366],
                                        [0.01316, 0.01387, 0.01612, 0.01668, 0.01449],
                                        [0.01451, 0.01572, 0.01797, 0.01826, 0.01614],
                                    ],
                                    [
                                        [0.00731, 0.00910, 0.01382, 0.01457, 0.01018],
                                        [0.00743, 0.00900, 0.01303, 0.01360, 0.00988],
                                        [0.00817, 0.00959, 0.01342, 0.01411, 0.01052],
                                        [0.00927, 0.01064, 0.01440, 0.01499, 0.01148],
                                    ],
                                    [
                                        [0.00525, 0.00742, 0.01359, 0.01385, 0.00887],
                                        [0.00556, 0.00738, 0.01271, 0.01300, 0.00856],
                                        [0.00622, 0.00788, 0.01271, 0.01307, 0.00904],
                                        [0.00716, 0.00878, 0.01334, 0.01361, 0.00988],
                                    ],
                                ],
                                [
                                    [
                                        [0.01210, 0.01309, 0.01519, 0.01543, 0.01356],
                                        [0.01227, 0.01311, 0.01520, 0.01556, 0.01362],
                                        [0.01389, 0.01467, 0.01663, 0.01696, 0.01510],
                                        [0.01599, 0.01676, 0.01851, 0.01884, 0.01717],
                                    ],
                                    [
                                        [0.00732, 0.00902, 0.01334, 0.01399, 0.00996],
                                        [0.00760, 0.00911, 0.01294, 0.01350, 0.00990],
                                        [0.00867, 0.01007, 0.01367, 0.01414, 0.01088],
                                        [0.01000, 0.01134, 0.01499, 0.01520, 0.01226],
                                    ],
                                    [
                                        [0.00523, 0.00746, 0.01313, 0.01339, 0.00882],
                                        [0.00566, 0.00757, 0.01265, 0.01288, 0.00877],
                                        [0.00657, 0.00835, 0.01281, 0.01309, 0.00944],
                                        [0.00771, 0.00940, 0.01360, 0.01371, 0.01052],
                                    ],
                                ],
                                [
                                    [
                                        [0.00434, 0.00507, 0.00568, 0.00583, 0.00534],
                                        [0.00567, 0.00645, 0.00705, 0.00721, 0.00671],
                                        [0.00742, 0.00819, 0.00871, 0.00880, 0.00838],
                                        [0.00954, 0.01034, 0.01069, 0.01089, 0.01049],
                                    ],
                                    [
                                        [0.00238, 0.00382, 0.00439, 0.00438, 0.00422],
                                        [0.00326, 0.00458, 0.00527, 0.00528, 0.00502],
                                        [0.00437, 0.00555, 0.00624, 0.00621, 0.00595],
                                        [0.00565, 0.00677, 0.00738, 0.00733, 0.00711],
                                    ],
                                    [
                                        [0.00156, 0.00315, 0.00385, 0.00387, 0.00354],
                                        [0.00229, 0.00366, 0.00454, 0.00458, 0.00412],
                                        [0.00319, 0.00439, 0.00525, 0.00526, 0.00480],
                                        [0.00424, 0.00531, 0.00610, 0.00606, 0.00567],
                                    ],
                                ],
                            ]
                        ),
                    },
                    {
                        "name": 64,
                        "particle": "ELECTRON",
                        "geom": "XYZ",
                        "bins": {
                            "ENERGY": np.array([1.00e-03, 1.00e02]),
                            "X": np.array([-3.00, -1.00, 1.00, 3.00]),
                            "Y": np.array([-3.00, -1.00, 1.00, 3.00, 5.00]),
                            "Z": np.array([-4.00, -3.00, -2.00, -1.00, 0.00, 1.00]),
                        },
                        "result": np.array(
                            [
                                [
                                    [
                                        [
                                            2.27419e-05,
                                            2.17868e-05,
                                            1.72577e-05,
                                            1.53648e-05,
                                            1.22356e-05,
                                        ],
                                        [
                                            3.11127e-05,
                                            2.85558e-05,
                                            2.31183e-05,
                                            1.84432e-05,
                                            1.36892e-05,
                                        ],
                                        [
                                            2.26343e-05,
                                            2.23131e-05,
                                            1.89640e-05,
                                            1.47211e-05,
                                            1.28286e-05,
                                        ],
                                        [
                                            1.07191e-05,
                                            1.14047e-05,
                                            1.02882e-05,
                                            9.13652e-06,
                                            8.13062e-06,
                                        ],
                                    ],
                                    [
                                        [
                                            3.05415e-05,
                                            2.95938e-05,
                                            2.37784e-05,
                                            1.72760e-05,
                                            1.26062e-05,
                                        ],
                                        [
                                            4.66317e-05,
                                            4.01764e-05,
                                            2.83337e-05,
                                            2.08623e-05,
                                            1.43317e-05,
                                        ],
                                        [
                                            3.00266e-05,
                                            2.91989e-05,
                                            2.23563e-05,
                                            1.74685e-05,
                                            1.32246e-05,
                                        ],
                                        [
                                            1.29678e-05,
                                            1.47402e-05,
                                            1.37617e-05,
                                            1.11206e-05,
                                            9.89096e-06,
                                        ],
                                    ],
                                    [
                                        [
                                            2.18894e-05,
                                            2.16545e-05,
                                            1.91636e-05,
                                            1.47231e-05,
                                            1.26959e-05,
                                        ],
                                        [
                                            2.88450e-05,
                                            2.85265e-05,
                                            2.21880e-05,
                                            1.67353e-05,
                                            1.43432e-05,
                                        ],
                                        [
                                            2.19202e-05,
                                            2.23430e-05,
                                            1.81618e-05,
                                            1.49328e-05,
                                            1.10308e-05,
                                        ],
                                        [
                                            1.11572e-05,
                                            1.18185e-05,
                                            1.00818e-05,
                                            9.17390e-06,
                                            7.42903e-06,
                                        ],
                                    ],
                                ]
                            ]
                        ),
                        "error": np.array(
                            [
                                [
                                    [
                                        [
                                            2.78362e-02,
                                            2.81298e-02,
                                            3.09657e-02,
                                            3.30592e-02,
                                            3.72776e-02,
                                        ],
                                        [
                                            2.36916e-02,
                                            2.42652e-02,
                                            2.71698e-02,
                                            3.04575e-02,
                                            3.49528e-02,
                                        ],
                                        [
                                            2.76754e-02,
                                            2.76985e-02,
                                            3.02390e-02,
                                            3.40211e-02,
                                            3.70110e-02,
                                        ],
                                        [
                                            4.23854e-02,
                                            4.03991e-02,
                                            4.24013e-02,
                                            4.51811e-02,
                                            4.82304e-02,
                                        ],
                                    ],
                                    [
                                        [
                                            2.36816e-02,
                                            2.44779e-02,
                                            2.70306e-02,
                                            3.09652e-02,
                                            3.63388e-02,
                                        ],
                                        [
                                            1.94626e-02,
                                            2.09634e-02,
                                            2.45109e-02,
                                            2.89800e-02,
                                            3.39383e-02,
                                        ],
                                        [
                                            2.40983e-02,
                                            2.44504e-02,
                                            2.73977e-02,
                                            3.12602e-02,
                                            3.52662e-02,
                                        ],
                                        [
                                            3.67117e-02,
                                            3.42116e-02,
                                            3.58161e-02,
                                            3.92775e-02,
                                            4.15590e-02,
                                        ],
                                    ],
                                    [
                                        [
                                            2.83303e-02,
                                            2.81832e-02,
                                            3.00929e-02,
                                            3.37627e-02,
                                            3.69545e-02,
                                        ],
                                        [
                                            2.45929e-02,
                                            2.47182e-02,
                                            2.79120e-02,
                                            3.20100e-02,
                                            3.41955e-02,
                                        ],
                                        [
                                            2.81071e-02,
                                            2.80327e-02,
                                            3.07090e-02,
                                            3.35533e-02,
                                            3.82874e-02,
                                        ],
                                        [
                                            4.15137e-02,
                                            3.92549e-02,
                                            4.21972e-02,
                                            4.47530e-02,
                                            5.03052e-02,
                                        ],
                                    ],
                                ]
                            ]
                        ),
                    },
                    {
                        "name": 74,
                        "particle": "NEUTRON",
                        "geom": "XYZ",
                        "bins": {
                            "ENERGY": np.array([0.00e00, 1.00e36]),
                            "X": np.array([-3.00, -1.00, 1.00, 3.00]),
                            "Y": np.array([-3.00, -1.00, 1.00, 3.00, 5.00]),
                            "Z": np.array([-4.00, -3.00, -2.00, -1.00, 0.00, 1.00]),
                        },
                        "result": np.array(
                            [
                                [
                                    [
                                        [
                                            9.04444e-03,
                                            6.27702e-03,
                                            4.35550e-03,
                                            3.08614e-03,
                                            2.24486e-03,
                                        ],
                                        [
                                            1.45306e-02,
                                            8.59736e-03,
                                            5.46472e-03,
                                            3.67699e-03,
                                            2.59166e-03,
                                        ],
                                        [
                                            9.03290e-03,
                                            6.26415e-03,
                                            4.33504e-03,
                                            3.08761e-03,
                                            2.23683e-03,
                                        ],
                                        [
                                            4.06754e-03,
                                            3.19242e-03,
                                            2.51993e-03,
                                            1.96192e-03,
                                            1.50871e-03,
                                        ],
                                    ],
                                    [
                                        [
                                            1.45132e-02,
                                            8.63447e-03,
                                            5.47173e-03,
                                            3.68526e-03,
                                            2.58725e-03,
                                        ],
                                        [
                                            3.19700e-02,
                                            1.31972e-02,
                                            7.10455e-03,
                                            4.40634e-03,
                                            2.96430e-03,
                                        ],
                                        [
                                            1.45048e-02,
                                            8.63950e-03,
                                            5.46359e-03,
                                            3.67178e-03,
                                            2.57053e-03,
                                        ],
                                        [
                                            4.99942e-03,
                                            3.89479e-03,
                                            2.99886e-03,
                                            2.26266e-03,
                                            1.73108e-03,
                                        ],
                                    ],
                                    [
                                        [
                                            9.08998e-03,
                                            6.27654e-03,
                                            4.38343e-03,
                                            3.11244e-03,
                                            2.25694e-03,
                                        ],
                                        [
                                            1.44999e-02,
                                            8.62357e-03,
                                            5.47940e-03,
                                            3.67164e-03,
                                            2.57404e-03,
                                        ],
                                        [
                                            9.09387e-03,
                                            6.25030e-03,
                                            4.35821e-03,
                                            3.10312e-03,
                                            2.25166e-03,
                                        ],
                                        [
                                            4.06678e-03,
                                            3.19813e-03,
                                            2.51588e-03,
                                            1.95568e-03,
                                            1.52583e-03,
                                        ],
                                    ],
                                ]
                            ]
                        ),
                        "error": np.array(
                            [
                                [
                                    [
                                        [
                                            1.97528e-03,
                                            2.24830e-03,
                                            2.63811e-03,
                                            3.10952e-03,
                                            3.63575e-03,
                                        ],
                                        [
                                            1.46500e-03,
                                            1.84926e-03,
                                            2.30835e-03,
                                            2.81878e-03,
                                            3.36778e-03,
                                        ],
                                        [
                                            1.97542e-03,
                                            2.25144e-03,
                                            2.64319e-03,
                                            3.10959e-03,
                                            3.62573e-03,
                                        ],
                                        [
                                            3.04694e-03,
                                            3.26414e-03,
                                            3.53917e-03,
                                            3.92537e-03,
                                            4.38197e-03,
                                        ],
                                    ],
                                    [
                                        [
                                            1.46549e-03,
                                            1.84509e-03,
                                            2.30611e-03,
                                            2.81385e-03,
                                            3.36711e-03,
                                        ],
                                        [
                                            8.39012e-04,
                                            1.39430e-03,
                                            1.95952e-03,
                                            2.53111e-03,
                                            3.11841e-03,
                                        ],
                                        [
                                            1.46653e-03,
                                            1.84612e-03,
                                            2.30960e-03,
                                            2.81762e-03,
                                            3.37416e-03,
                                        ],
                                        [
                                            2.73104e-03,
                                            2.95514e-03,
                                            3.25122e-03,
                                            3.66135e-03,
                                            4.13756e-03,
                                        ],
                                    ],
                                    [
                                        [
                                            1.97264e-03,
                                            2.24742e-03,
                                            2.63617e-03,
                                            3.08949e-03,
                                            3.61779e-03,
                                        ],
                                        [
                                            1.46565e-03,
                                            1.84595e-03,
                                            2.30669e-03,
                                            2.81720e-03,
                                            3.37333e-03,
                                        ],
                                        [
                                            1.97189e-03,
                                            2.25217e-03,
                                            2.63446e-03,
                                            3.09805e-03,
                                            3.62754e-03,
                                        ],
                                        [
                                            3.04754e-03,
                                            3.26402e-03,
                                            3.54243e-03,
                                            3.91995e-03,
                                            4.37779e-03,
                                        ],
                                    ],
                                ]
                            ]
                        ),
                    },
                ],
            },
        ),
        (
            "parser_test_data/d1s_mesh.m",
            {
                "date": "12/17/1814:06:18",
                "histories": 766606920.00,
                "title": "Test of neutron flux calculations for reference.",
                "tallies": [
                    {
                        "name": 14,
                        "particle": "PHOTON",
                        "geom": "XYZ",
                        "bins": {
                            "TIME": np.array([1.00e00, 2.00e00, 3.00e00, 4.00e00]),
                            "X": np.array([-5.00, -3.00, -1.00, 1.00, 3.00, 5.00]),
                            "Y": np.array([-5.00, -3.00, -1.00, 1.00, 3.00, 5.00]),
                            "Z": np.array([-5.00, -3.00, -1.00, 1.00, 3.00, 5.00]),
                        },
                        "result": np.array(
                            [
                                [
                                    [
                                        [
                                            8.92453e-08,
                                            1.24887e-07,
                                            1.56804e-07,
                                            1.61620e-07,
                                            1.34217e-07,
                                        ],
                                        [
                                            1.10391e-07,
                                            1.77126e-07,
                                            2.51215e-07,
                                            2.62388e-07,
                                            1.95196e-07,
                                        ],
                                        [
                                            1.23021e-07,
                                            2.08444e-07,
                                            3.18043e-07,
                                            3.29838e-07,
                                            2.29984e-07,
                                        ],
                                        [
                                            1.12524e-07,
                                            1.76263e-07,
                                            2.55344e-07,
                                            2.64600e-07,
                                            1.93915e-07,
                                        ],
                                        [
                                            8.96732e-08,
                                            1.26210e-07,
                                            1.57158e-07,
                                            1.61266e-07,
                                            1.35721e-07,
                                        ],
                                    ],
                                    [
                                        [
                                            1.13300e-07,
                                            1.77945e-07,
                                            2.46814e-07,
                                            2.59300e-07,
                                            1.93870e-07,
                                        ],
                                        [
                                            1.52459e-07,
                                            3.06716e-07,
                                            7.14723e-07,
                                            7.19620e-07,
                                            3.77493e-07,
                                        ],
                                        [
                                            1.72072e-07,
                                            3.91336e-07,
                                            1.46341e-06,
                                            1.38556e-06,
                                            5.48045e-07,
                                        ],
                                        [
                                            1.53821e-07,
                                            3.07244e-07,
                                            7.22951e-07,
                                            7.25258e-07,
                                            3.78795e-07,
                                        ],
                                        [
                                            1.11756e-07,
                                            1.81490e-07,
                                            2.53272e-07,
                                            2.61106e-07,
                                            1.94527e-07,
                                        ],
                                    ],
                                    [
                                        [
                                            1.25950e-07,
                                            2.11558e-07,
                                            3.16217e-07,
                                            3.28060e-07,
                                            2.33868e-07,
                                        ],
                                        [
                                            1.71856e-07,
                                            3.96604e-07,
                                            1.45958e-06,
                                            1.38734e-06,
                                            5.48853e-07,
                                        ],
                                        [
                                            1.96700e-07,
                                            5.29040e-07,
                                            2.73243e-06,
                                            2.53523e-06,
                                            8.33311e-07,
                                        ],
                                        [
                                            1.72650e-07,
                                            3.96096e-07,
                                            1.46463e-06,
                                            1.38428e-06,
                                            5.47845e-07,
                                        ],
                                        [
                                            1.24008e-07,
                                            2.07453e-07,
                                            3.19641e-07,
                                            3.30529e-07,
                                            2.32478e-07,
                                        ],
                                    ],
                                    [
                                        [
                                            1.13063e-07,
                                            1.79890e-07,
                                            2.54586e-07,
                                            2.63512e-07,
                                            1.95776e-07,
                                        ],
                                        [
                                            1.53673e-07,
                                            3.07348e-07,
                                            7.20651e-07,
                                            7.27094e-07,
                                            3.79922e-07,
                                        ],
                                        [
                                            1.70691e-07,
                                            3.95201e-07,
                                            1.46635e-06,
                                            1.38597e-06,
                                            5.47437e-07,
                                        ],
                                        [
                                            1.52316e-07,
                                            3.08879e-07,
                                            7.21570e-07,
                                            7.23136e-07,
                                            3.77177e-07,
                                        ],
                                        [
                                            1.13012e-07,
                                            1.78413e-07,
                                            2.50868e-07,
                                            2.62647e-07,
                                            1.93352e-07,
                                        ],
                                    ],
                                    [
                                        [
                                            9.06897e-08,
                                            1.27240e-07,
                                            1.56990e-07,
                                            1.59999e-07,
                                            1.32088e-07,
                                        ],
                                        [
                                            1.14633e-07,
                                            1.79672e-07,
                                            2.50255e-07,
                                            2.63184e-07,
                                            1.95092e-07,
                                        ],
                                        [
                                            1.22807e-07,
                                            2.06967e-07,
                                            3.18653e-07,
                                            3.30730e-07,
                                            2.31173e-07,
                                        ],
                                        [
                                            1.13695e-07,
                                            1.81119e-07,
                                            2.53129e-07,
                                            2.64267e-07,
                                            1.93465e-07,
                                        ],
                                        [
                                            8.95939e-08,
                                            1.26791e-07,
                                            1.58979e-07,
                                            1.61257e-07,
                                            1.33504e-07,
                                        ],
                                    ],
                                ],
                                [
                                    [
                                        [
                                            8.52058e-08,
                                            1.19234e-07,
                                            1.49707e-07,
                                            1.54304e-07,
                                            1.28142e-07,
                                        ],
                                        [
                                            1.05394e-07,
                                            1.69109e-07,
                                            2.39844e-07,
                                            2.50512e-07,
                                            1.86361e-07,
                                        ],
                                        [
                                            1.17453e-07,
                                            1.99010e-07,
                                            3.03648e-07,
                                            3.14908e-07,
                                            2.19574e-07,
                                        ],
                                        [
                                            1.07431e-07,
                                            1.68284e-07,
                                            2.43786e-07,
                                            2.52623e-07,
                                            1.85137e-07,
                                        ],
                                        [
                                            8.56144e-08,
                                            1.20497e-07,
                                            1.50045e-07,
                                            1.53967e-07,
                                            1.29578e-07,
                                        ],
                                    ],
                                    [
                                        [
                                            1.08171e-07,
                                            1.69891e-07,
                                            2.35642e-07,
                                            2.47564e-07,
                                            1.85095e-07,
                                        ],
                                        [
                                            1.45558e-07,
                                            2.92833e-07,
                                            6.82372e-07,
                                            6.87048e-07,
                                            3.60407e-07,
                                        ],
                                        [
                                            1.64283e-07,
                                            3.73623e-07,
                                            1.39717e-06,
                                            1.32284e-06,
                                            5.23239e-07,
                                        ],
                                        [
                                            1.46858e-07,
                                            2.93337e-07,
                                            6.90228e-07,
                                            6.92431e-07,
                                            3.61649e-07,
                                        ],
                                        [
                                            1.06697e-07,
                                            1.73275e-07,
                                            2.41808e-07,
                                            2.49288e-07,
                                            1.85722e-07,
                                        ],
                                    ],
                                    [
                                        [
                                            1.20249e-07,
                                            2.01982e-07,
                                            3.01904e-07,
                                            3.13211e-07,
                                            2.23282e-07,
                                        ],
                                        [
                                            1.64077e-07,
                                            3.78653e-07,
                                            1.39351e-06,
                                            1.32454e-06,
                                            5.24011e-07,
                                        ],
                                        [
                                            1.87796e-07,
                                            5.05094e-07,
                                            2.60875e-06,
                                            2.42048e-06,
                                            7.95593e-07,
                                        ],
                                        [
                                            1.64836e-07,
                                            3.78167e-07,
                                            1.39834e-06,
                                            1.32162e-06,
                                            5.23048e-07,
                                        ],
                                        [
                                            1.18395e-07,
                                            1.98063e-07,
                                            3.05173e-07,
                                            3.15569e-07,
                                            2.21955e-07,
                                        ],
                                    ],
                                    [
                                        [
                                            1.07946e-07,
                                            1.71748e-07,
                                            2.43062e-07,
                                            2.51585e-07,
                                            1.86915e-07,
                                        ],
                                        [
                                            1.46717e-07,
                                            2.93437e-07,
                                            6.88032e-07,
                                            6.94184e-07,
                                            3.62726e-07,
                                        ],
                                        [
                                            1.62965e-07,
                                            3.77313e-07,
                                            1.39998e-06,
                                            1.32324e-06,
                                            5.22658e-07,
                                        ],
                                        [
                                            1.45422e-07,
                                            2.94898e-07,
                                            6.88910e-07,
                                            6.90405e-07,
                                            3.60105e-07,
                                        ],
                                        [
                                            1.07897e-07,
                                            1.70338e-07,
                                            2.39513e-07,
                                            2.50759e-07,
                                            1.84600e-07,
                                        ],
                                    ],
                                    [
                                        [
                                            8.65848e-08,
                                            1.21481e-07,
                                            1.49884e-07,
                                            1.52757e-07,
                                            1.26109e-07,
                                        ],
                                        [
                                            1.09445e-07,
                                            1.71540e-07,
                                            2.38928e-07,
                                            2.51272e-07,
                                            1.86262e-07,
                                        ],
                                        [
                                            1.17249e-07,
                                            1.97599e-07,
                                            3.04230e-07,
                                            3.15761e-07,
                                            2.20709e-07,
                                        ],
                                        [
                                            1.08549e-07,
                                            1.72921e-07,
                                            2.41671e-07,
                                            2.52306e-07,
                                            1.84709e-07,
                                        ],
                                        [
                                            8.55387e-08,
                                            1.21052e-07,
                                            1.51783e-07,
                                            1.53958e-07,
                                            1.27461e-07,
                                        ],
                                    ],
                                ],
                                [
                                    [
                                        [
                                            5.61531e-08,
                                            7.85788e-08,
                                            9.86612e-08,
                                            1.01691e-07,
                                            8.44495e-08,
                                        ],
                                        [
                                            6.94578e-08,
                                            1.11447e-07,
                                            1.58064e-07,
                                            1.65095e-07,
                                            1.22817e-07,
                                        ],
                                        [
                                            7.74048e-08,
                                            1.31153e-07,
                                            2.00113e-07,
                                            2.07534e-07,
                                            1.44705e-07,
                                        ],
                                        [
                                            7.08003e-08,
                                            1.10904e-07,
                                            1.60662e-07,
                                            1.66486e-07,
                                            1.22011e-07,
                                        ],
                                        [
                                            5.64223e-08,
                                            7.94112e-08,
                                            9.88838e-08,
                                            1.01468e-07,
                                            8.53958e-08,
                                        ],
                                    ],
                                    [
                                        [
                                            7.12881e-08,
                                            1.11963e-07,
                                            1.55295e-07,
                                            1.63152e-07,
                                            1.21983e-07,
                                        ],
                                        [
                                            9.59271e-08,
                                            1.92985e-07,
                                            4.49703e-07,
                                            4.52785e-07,
                                            2.37519e-07,
                                        ],
                                        [
                                            1.08267e-07,
                                            2.46228e-07,
                                            9.20774e-07,
                                            8.71792e-07,
                                            3.44830e-07,
                                        ],
                                        [
                                            9.67838e-08,
                                            1.93318e-07,
                                            4.54880e-07,
                                            4.56332e-07,
                                            2.38337e-07,
                                        ],
                                        [
                                            7.03165e-08,
                                            1.14193e-07,
                                            1.59358e-07,
                                            1.64288e-07,
                                            1.22396e-07,
                                        ],
                                    ],
                                    [
                                        [
                                            7.92476e-08,
                                            1.33112e-07,
                                            1.98963e-07,
                                            2.06415e-07,
                                            1.47149e-07,
                                        ],
                                        [
                                            1.08132e-07,
                                            2.49543e-07,
                                            9.18364e-07,
                                            8.72912e-07,
                                            3.45338e-07,
                                        ],
                                        [
                                            1.23763e-07,
                                            3.32872e-07,
                                            1.71924e-06,
                                            1.59516e-06,
                                            5.24318e-07,
                                        ],
                                        [
                                            1.08631e-07,
                                            2.49223e-07,
                                            9.21543e-07,
                                            8.70986e-07,
                                            3.44704e-07,
                                        ],
                                        [
                                            7.80260e-08,
                                            1.30529e-07,
                                            2.01118e-07,
                                            2.07969e-07,
                                            1.46275e-07,
                                        ],
                                    ],
                                    [
                                        [
                                            7.11394e-08,
                                            1.13187e-07,
                                            1.60185e-07,
                                            1.65802e-07,
                                            1.23182e-07,
                                        ],
                                        [
                                            9.66908e-08,
                                            1.93383e-07,
                                            4.53433e-07,
                                            4.57487e-07,
                                            2.39047e-07,
                                        ],
                                        [
                                            1.07398e-07,
                                            2.48660e-07,
                                            9.22626e-07,
                                            8.72050e-07,
                                            3.44447e-07,
                                        ],
                                        [
                                            9.58374e-08,
                                            1.94346e-07,
                                            4.54011e-07,
                                            4.54997e-07,
                                            2.37319e-07,
                                        ],
                                        [
                                            7.11071e-08,
                                            1.12257e-07,
                                            1.57846e-07,
                                            1.65258e-07,
                                            1.21657e-07,
                                        ],
                                    ],
                                    [
                                        [
                                            5.70619e-08,
                                            8.00594e-08,
                                            9.87782e-08,
                                            1.00671e-07,
                                            8.31096e-08,
                                        ],
                                        [
                                            7.21273e-08,
                                            1.13050e-07,
                                            1.57460e-07,
                                            1.65595e-07,
                                            1.22752e-07,
                                        ],
                                        [
                                            7.72703e-08,
                                            1.30224e-07,
                                            2.00496e-07,
                                            2.08095e-07,
                                            1.45454e-07,
                                        ],
                                        [
                                            7.15368e-08,
                                            1.13960e-07,
                                            1.59268e-07,
                                            1.66277e-07,
                                            1.21728e-07,
                                        ],
                                        [
                                            5.63725e-08,
                                            7.97769e-08,
                                            1.00029e-07,
                                            1.01463e-07,
                                            8.40004e-08,
                                        ],
                                    ],
                                ],
                                [
                                    [
                                        [
                                            1.84625e-08,
                                            2.58358e-08,
                                            3.24387e-08,
                                            3.34348e-08,
                                            2.77660e-08,
                                        ],
                                        [
                                            2.28369e-08,
                                            3.66426e-08,
                                            5.19697e-08,
                                            5.42812e-08,
                                            4.03810e-08,
                                        ],
                                        [
                                            2.54498e-08,
                                            4.31216e-08,
                                            6.57947e-08,
                                            6.82347e-08,
                                            4.75775e-08,
                                        ],
                                        [
                                            2.32783e-08,
                                            3.64640e-08,
                                            5.28238e-08,
                                            5.47386e-08,
                                            4.01158e-08,
                                        ],
                                        [
                                            1.85510e-08,
                                            2.61095e-08,
                                            3.25118e-08,
                                            3.33616e-08,
                                            2.80771e-08,
                                        ],
                                    ],
                                    [
                                        [
                                            2.34387e-08,
                                            3.68121e-08,
                                            5.10592e-08,
                                            5.36423e-08,
                                            4.01065e-08,
                                        ],
                                        [
                                            3.15397e-08,
                                            6.34514e-08,
                                            1.47857e-07,
                                            1.48870e-07,
                                            7.80933e-08,
                                        ],
                                        [
                                            3.55971e-08,
                                            8.09570e-08,
                                            3.02740e-07,
                                            2.86635e-07,
                                            1.13376e-07,
                                        ],
                                        [
                                            3.18214e-08,
                                            6.35606e-08,
                                            1.49559e-07,
                                            1.50037e-07,
                                            7.83626e-08,
                                        ],
                                        [
                                            2.31192e-08,
                                            3.75454e-08,
                                            5.23952e-08,
                                            5.40159e-08,
                                            4.02425e-08,
                                        ],
                                    ],
                                    [
                                        [
                                            2.60557e-08,
                                            4.37657e-08,
                                            6.54168e-08,
                                            6.78669e-08,
                                            4.83810e-08,
                                        ],
                                        [
                                            3.55524e-08,
                                            8.20468e-08,
                                            3.01947e-07,
                                            2.87003e-07,
                                            1.13543e-07,
                                        ],
                                        [
                                            4.06919e-08,
                                            1.09444e-07,
                                            5.65266e-07,
                                            5.24471e-07,
                                            1.72390e-07,
                                        ],
                                        [
                                            3.57167e-08,
                                            8.19417e-08,
                                            3.02993e-07,
                                            2.86370e-07,
                                            1.13335e-07,
                                        ],
                                        [
                                            2.56540e-08,
                                            4.29166e-08,
                                            6.61251e-08,
                                            6.83778e-08,
                                            4.80934e-08,
                                        ],
                                    ],
                                    [
                                        [
                                            2.33898e-08,
                                            3.72146e-08,
                                            5.26670e-08,
                                            5.45137e-08,
                                            4.05009e-08,
                                        ],
                                        [
                                            3.17908e-08,
                                            6.35822e-08,
                                            1.49083e-07,
                                            1.50416e-07,
                                            7.85958e-08,
                                        ],
                                        [
                                            3.53113e-08,
                                            8.17565e-08,
                                            3.03349e-07,
                                            2.86720e-07,
                                            1.13250e-07,
                                        ],
                                        [
                                            3.15102e-08,
                                            6.38988e-08,
                                            1.49274e-07,
                                            1.49598e-07,
                                            7.80278e-08,
                                        ],
                                        [
                                            2.33792e-08,
                                            3.69089e-08,
                                            5.18979e-08,
                                            5.43348e-08,
                                            3.99993e-08,
                                        ],
                                    ],
                                    [
                                        [
                                            1.87613e-08,
                                            2.63226e-08,
                                            3.24771e-08,
                                            3.30995e-08,
                                            2.73255e-08,
                                        ],
                                        [
                                            2.37146e-08,
                                            3.71695e-08,
                                            5.17712e-08,
                                            5.44458e-08,
                                            4.03593e-08,
                                        ],
                                        [
                                            2.54056e-08,
                                            4.28160e-08,
                                            6.59208e-08,
                                            6.84193e-08,
                                            4.78235e-08,
                                        ],
                                        [
                                            2.35205e-08,
                                            3.74687e-08,
                                            5.23656e-08,
                                            5.46699e-08,
                                            4.00229e-08,
                                        ],
                                        [
                                            1.85346e-08,
                                            2.62297e-08,
                                            3.28884e-08,
                                            3.33598e-08,
                                            2.76184e-08,
                                        ],
                                    ],
                                ],
                            ]
                        ),
                        "error": np.array(
                            [
                                [
                                    [
                                        [0.01163, 0.00982, 0.00880, 0.00872, 0.00951],
                                        [0.01048, 0.00830, 0.00694, 0.00683, 0.00792],
                                        [0.00999, 0.00770, 0.00614, 0.00605, 0.00728],
                                        [0.01039, 0.00832, 0.00691, 0.00680, 0.00794],
                                        [0.01156, 0.00977, 0.00879, 0.00871, 0.00946],
                                    ],
                                    [
                                        [0.01039, 0.00829, 0.00701, 0.00687, 0.00795],
                                        [0.00896, 0.00634, 0.00404, 0.00410, 0.00573],
                                        [0.00845, 0.00557, 0.00274, 0.00288, 0.00471],
                                        [0.00891, 0.00634, 0.00402, 0.00408, 0.00571],
                                        [0.01043, 0.00824, 0.00692, 0.00685, 0.00792],
                                    ],
                                    [
                                        [0.00989, 0.00764, 0.00614, 0.00605, 0.00724],
                                        [0.00844, 0.00554, 0.00274, 0.00288, 0.00470],
                                        [0.00790, 0.00476, 0.00200, 0.00214, 0.00380],
                                        [0.00843, 0.00554, 0.00274, 0.00288, 0.00471],
                                        [0.01000, 0.00769, 0.00612, 0.00604, 0.00726],
                                    ],
                                    [
                                        [0.01036, 0.00824, 0.00691, 0.00682, 0.00790],
                                        [0.00892, 0.00632, 0.00403, 0.00407, 0.00571],
                                        [0.00848, 0.00555, 0.00274, 0.00288, 0.00472],
                                        [0.00897, 0.00631, 0.00402, 0.00408, 0.00572],
                                        [0.01039, 0.00827, 0.00696, 0.00682, 0.00794],
                                    ],
                                    [
                                        [0.01152, 0.00976, 0.00881, 0.00875, 0.00957],
                                        [0.01031, 0.00825, 0.00697, 0.00682, 0.00791],
                                        [0.01002, 0.00769, 0.00612, 0.00603, 0.00727],
                                        [0.01037, 0.00821, 0.00692, 0.00680, 0.00795],
                                        [0.01159, 0.00976, 0.00875, 0.00870, 0.00954],
                                    ],
                                ],
                                [
                                    [
                                        [0.01163, 0.00982, 0.00880, 0.00872, 0.00951],
                                        [0.01048, 0.00830, 0.00694, 0.00683, 0.00792],
                                        [0.00999, 0.00770, 0.00614, 0.00605, 0.00728],
                                        [0.01039, 0.00832, 0.00691, 0.00680, 0.00794],
                                        [0.01156, 0.00977, 0.00879, 0.00871, 0.00946],
                                    ],
                                    [
                                        [0.01039, 0.00829, 0.00701, 0.00687, 0.00795],
                                        [0.00896, 0.00634, 0.00404, 0.00410, 0.00573],
                                        [0.00845, 0.00557, 0.00274, 0.00288, 0.00471],
                                        [0.00891, 0.00634, 0.00402, 0.00408, 0.00571],
                                        [0.01043, 0.00824, 0.00692, 0.00685, 0.00792],
                                    ],
                                    [
                                        [0.00989, 0.00764, 0.00614, 0.00605, 0.00724],
                                        [0.00844, 0.00554, 0.00274, 0.00288, 0.00470],
                                        [0.00790, 0.00476, 0.00200, 0.00214, 0.00380],
                                        [0.00843, 0.00554, 0.00274, 0.00288, 0.00471],
                                        [0.01000, 0.00769, 0.00612, 0.00604, 0.00726],
                                    ],
                                    [
                                        [0.01036, 0.00824, 0.00691, 0.00682, 0.00790],
                                        [0.00892, 0.00632, 0.00403, 0.00407, 0.00571],
                                        [0.00848, 0.00555, 0.00274, 0.00288, 0.00472],
                                        [0.00897, 0.00631, 0.00402, 0.00408, 0.00572],
                                        [0.01039, 0.00827, 0.00696, 0.00682, 0.00794],
                                    ],
                                    [
                                        [0.01152, 0.00976, 0.00881, 0.00875, 0.00957],
                                        [0.01031, 0.00825, 0.00697, 0.00682, 0.00791],
                                        [0.01002, 0.00769, 0.00612, 0.00603, 0.00727],
                                        [0.01037, 0.00821, 0.00692, 0.00680, 0.00795],
                                        [0.01159, 0.00976, 0.00875, 0.00870, 0.00954],
                                    ],
                                ],
                                [
                                    [
                                        [0.01163, 0.00982, 0.00880, 0.00872, 0.00951],
                                        [0.01048, 0.00830, 0.00694, 0.00683, 0.00792],
                                        [0.00999, 0.00770, 0.00614, 0.00605, 0.00728],
                                        [0.01039, 0.00832, 0.00691, 0.00680, 0.00794],
                                        [0.01156, 0.00977, 0.00879, 0.00871, 0.00946],
                                    ],
                                    [
                                        [0.01039, 0.00829, 0.00701, 0.00687, 0.00795],
                                        [0.00896, 0.00634, 0.00404, 0.00410, 0.00573],
                                        [0.00845, 0.00557, 0.00274, 0.00288, 0.00471],
                                        [0.00891, 0.00634, 0.00402, 0.00408, 0.00571],
                                        [0.01043, 0.00824, 0.00692, 0.00685, 0.00792],
                                    ],
                                    [
                                        [0.00989, 0.00764, 0.00614, 0.00605, 0.00724],
                                        [0.00844, 0.00554, 0.00274, 0.00288, 0.00470],
                                        [0.00790, 0.00476, 0.00200, 0.00214, 0.00380],
                                        [0.00843, 0.00554, 0.00274, 0.00288, 0.00471],
                                        [0.01000, 0.00769, 0.00612, 0.00604, 0.00726],
                                    ],
                                    [
                                        [0.01036, 0.00824, 0.00691, 0.00682, 0.00790],
                                        [0.00892, 0.00632, 0.00403, 0.00407, 0.00571],
                                        [0.00848, 0.00555, 0.00274, 0.00288, 0.00472],
                                        [0.00897, 0.00631, 0.00402, 0.00408, 0.00572],
                                        [0.01039, 0.00827, 0.00696, 0.00682, 0.00794],
                                    ],
                                    [
                                        [0.01152, 0.00976, 0.00881, 0.00875, 0.00957],
                                        [0.01031, 0.00825, 0.00697, 0.00682, 0.00791],
                                        [0.01002, 0.00769, 0.00612, 0.00603, 0.00727],
                                        [0.01037, 0.00821, 0.00692, 0.00680, 0.00795],
                                        [0.01159, 0.00976, 0.00875, 0.00870, 0.00954],
                                    ],
                                ],
                                [
                                    [
                                        [0.01163, 0.00982, 0.00880, 0.00872, 0.00951],
                                        [0.01048, 0.00830, 0.00694, 0.00683, 0.00792],
                                        [0.00999, 0.00770, 0.00614, 0.00605, 0.00728],
                                        [0.01039, 0.00832, 0.00691, 0.00680, 0.00794],
                                        [0.01156, 0.00977, 0.00879, 0.00871, 0.00946],
                                    ],
                                    [
                                        [0.01039, 0.00829, 0.00701, 0.00687, 0.00795],
                                        [0.00896, 0.00634, 0.00404, 0.00410, 0.00573],
                                        [0.00845, 0.00557, 0.00274, 0.00288, 0.00471],
                                        [0.00891, 0.00634, 0.00402, 0.00408, 0.00571],
                                        [0.01043, 0.00824, 0.00692, 0.00685, 0.00792],
                                    ],
                                    [
                                        [0.00989, 0.00764, 0.00614, 0.00605, 0.00724],
                                        [0.00844, 0.00554, 0.00274, 0.00288, 0.00470],
                                        [0.00790, 0.00476, 0.00200, 0.00214, 0.00380],
                                        [0.00843, 0.00554, 0.00274, 0.00288, 0.00471],
                                        [0.01000, 0.00769, 0.00612, 0.00604, 0.00726],
                                    ],
                                    [
                                        [0.01036, 0.00824, 0.00691, 0.00682, 0.00790],
                                        [0.00892, 0.00632, 0.00403, 0.00407, 0.00571],
                                        [0.00848, 0.00555, 0.00274, 0.00288, 0.00472],
                                        [0.00897, 0.00631, 0.00402, 0.00408, 0.00572],
                                        [0.01039, 0.00827, 0.00696, 0.00682, 0.00794],
                                    ],
                                    [
                                        [0.01152, 0.00976, 0.00881, 0.00875, 0.00957],
                                        [0.01031, 0.00825, 0.00697, 0.00682, 0.00791],
                                        [0.01002, 0.00769, 0.00612, 0.00603, 0.00727],
                                        [0.01037, 0.00821, 0.00692, 0.00680, 0.00795],
                                        [0.01159, 0.00976, 0.00875, 0.00870, 0.00954],
                                    ],
                                ],
                            ]
                        ),
                    },
                    {
                        "name": 24,
                        "particle": "PHOTON",
                        "geom": "XYZ",
                        "bins": {
                            "TIME": np.array([1.00e00, 2.00e00, 3.00e00, 4.00e00]),
                            "X": np.array([-5.00, -3.00, -1.00, 1.00, 3.00, 5.00]),
                            "Y": np.array([-5.00, -3.00, -1.00, 1.00, 3.00, 5.00]),
                            "Z": np.array([-5.00, -3.00, -1.00, 1.00, 3.00, 5.00]),
                        },
                        "result": np.array(
                            [
                                [
                                    [
                                        [
                                            1.65147e-09,
                                            2.20534e-09,
                                            2.70833e-09,
                                            2.85064e-09,
                                            2.34434e-09,
                                        ],
                                        [
                                            1.87587e-09,
                                            3.02620e-09,
                                            4.19142e-09,
                                            4.65720e-09,
                                            3.38723e-09,
                                        ],
                                        [
                                            2.12946e-09,
                                            3.53725e-09,
                                            5.39405e-09,
                                            5.94829e-09,
                                            3.99410e-09,
                                        ],
                                        [
                                            2.01459e-09,
                                            2.98621e-09,
                                            4.37829e-09,
                                            4.67520e-09,
                                            3.34176e-09,
                                        ],
                                        [
                                            1.60243e-09,
                                            2.18338e-09,
                                            2.62385e-09,
                                            2.98216e-09,
                                            2.40741e-09,
                                        ],
                                    ],
                                    [
                                        [
                                            1.94392e-09,
                                            3.07505e-09,
                                            4.24377e-09,
                                            4.55821e-09,
                                            3.41581e-09,
                                        ],
                                        [
                                            2.72289e-09,
                                            5.29737e-09,
                                            1.19796e-08,
                                            1.29272e-08,
                                            6.49600e-09,
                                        ],
                                        [
                                            2.98523e-09,
                                            6.77768e-09,
                                            2.47736e-08,
                                            2.61942e-08,
                                            9.54929e-09,
                                        ],
                                        [
                                            2.70996e-09,
                                            5.35058e-09,
                                            1.20420e-08,
                                            1.30299e-08,
                                            6.53883e-09,
                                        ],
                                        [
                                            1.94392e-09,
                                            3.19938e-09,
                                            4.35328e-09,
                                            4.52467e-09,
                                            3.34305e-09,
                                        ],
                                    ],
                                    [
                                        [
                                            2.15906e-09,
                                            3.66856e-09,
                                            5.37024e-09,
                                            5.69757e-09,
                                            4.18130e-09,
                                        ],
                                        [
                                            3.00804e-09,
                                            6.73148e-09,
                                            2.45272e-08,
                                            2.61140e-08,
                                            9.75044e-09,
                                        ],
                                        [
                                            3.49464e-09,
                                            9.36023e-09,
                                            4.96824e-08,
                                            5.22206e-08,
                                            1.50825e-08,
                                        ],
                                        [
                                            3.02436e-09,
                                            6.84773e-09,
                                            2.48619e-08,
                                            2.61678e-08,
                                            9.77084e-09,
                                        ],
                                        [
                                            2.17167e-09,
                                            3.65755e-09,
                                            5.46897e-09,
                                            5.79175e-09,
                                            4.26473e-09,
                                        ],
                                    ],
                                    [
                                        [
                                            1.94598e-09,
                                            3.08064e-09,
                                            4.28636e-09,
                                            4.75630e-09,
                                            3.45892e-09,
                                        ],
                                        [
                                            2.69612e-09,
                                            5.43925e-09,
                                            1.22050e-08,
                                            1.31705e-08,
                                            6.48057e-09,
                                        ],
                                        [
                                            3.10190e-09,
                                            6.97084e-09,
                                            2.51991e-08,
                                            2.61734e-08,
                                            9.48048e-09,
                                        ],
                                        [
                                            2.68530e-09,
                                            5.32297e-09,
                                            1.21761e-08,
                                            1.30855e-08,
                                            6.51168e-09,
                                        ],
                                        [
                                            2.01131e-09,
                                            3.05422e-09,
                                            4.24709e-09,
                                            4.67083e-09,
                                            3.42486e-09,
                                        ],
                                    ],
                                    [
                                        [
                                            1.61981e-09,
                                            2.22746e-09,
                                            2.64450e-09,
                                            2.76347e-09,
                                            2.37856e-09,
                                        ],
                                        [
                                            2.05575e-09,
                                            3.15936e-09,
                                            4.25688e-09,
                                            4.73740e-09,
                                            3.37773e-09,
                                        ],
                                        [
                                            2.17750e-09,
                                            3.72092e-09,
                                            5.44290e-09,
                                            5.80180e-09,
                                            3.98859e-09,
                                        ],
                                        [
                                            1.97145e-09,
                                            3.16050e-09,
                                            4.45975e-09,
                                            4.60044e-09,
                                            3.29786e-09,
                                        ],
                                        [
                                            1.55682e-09,
                                            2.22582e-09,
                                            2.84455e-09,
                                            2.79964e-09,
                                            2.29651e-09,
                                        ],
                                    ],
                                ],
                                [
                                    [
                                        [
                                            1.57672e-09,
                                            2.10552e-09,
                                            2.58574e-09,
                                            2.72162e-09,
                                            2.23823e-09,
                                        ],
                                        [
                                            1.79097e-09,
                                            2.88923e-09,
                                            4.00170e-09,
                                            4.44640e-09,
                                            3.23391e-09,
                                        ],
                                        [
                                            2.03308e-09,
                                            3.37714e-09,
                                            5.14990e-09,
                                            5.67906e-09,
                                            3.81331e-09,
                                        ],
                                        [
                                            1.92340e-09,
                                            2.85104e-09,
                                            4.18011e-09,
                                            4.46359e-09,
                                            3.19051e-09,
                                        ],
                                        [
                                            1.52990e-09,
                                            2.08456e-09,
                                            2.50509e-09,
                                            2.84717e-09,
                                            2.29844e-09,
                                        ],
                                    ],
                                    [
                                        [
                                            1.85593e-09,
                                            2.93586e-09,
                                            4.05168e-09,
                                            4.35189e-09,
                                            3.26120e-09,
                                        ],
                                        [
                                            2.59964e-09,
                                            5.05759e-09,
                                            1.14374e-08,
                                            1.23421e-08,
                                            6.20197e-09,
                                        ],
                                        [
                                            2.85011e-09,
                                            6.47090e-09,
                                            2.36522e-08,
                                            2.50086e-08,
                                            9.11706e-09,
                                        ],
                                        [
                                            2.58730e-09,
                                            5.10840e-09,
                                            1.14970e-08,
                                            1.24401e-08,
                                            6.24287e-09,
                                        ],
                                        [
                                            1.85593e-09,
                                            3.05457e-09,
                                            4.15624e-09,
                                            4.31987e-09,
                                            3.19173e-09,
                                        ],
                                    ],
                                    [
                                        [
                                            2.06134e-09,
                                            3.50251e-09,
                                            5.12716e-09,
                                            5.43968e-09,
                                            3.99204e-09,
                                        ],
                                        [
                                            2.87189e-09,
                                            6.42679e-09,
                                            2.34170e-08,
                                            2.49320e-08,
                                            9.30911e-09,
                                        ],
                                        [
                                            3.33646e-09,
                                            8.93656e-09,
                                            4.74336e-08,
                                            4.98570e-08,
                                            1.43999e-08,
                                        ],
                                        [
                                            2.88747e-09,
                                            6.53778e-09,
                                            2.37365e-08,
                                            2.49834e-08,
                                            9.32858e-09,
                                        ],
                                        [
                                            2.07337e-09,
                                            3.49200e-09,
                                            5.22143e-09,
                                            5.52960e-09,
                                            4.07170e-09,
                                        ],
                                    ],
                                    [
                                        [
                                            1.85790e-09,
                                            2.94120e-09,
                                            4.09235e-09,
                                            4.54102e-09,
                                            3.30236e-09,
                                        ],
                                        [
                                            2.57409e-09,
                                            5.19305e-09,
                                            1.16525e-08,
                                            1.25744e-08,
                                            6.18724e-09,
                                        ],
                                        [
                                            2.96150e-09,
                                            6.65532e-09,
                                            2.40585e-08,
                                            2.49887e-08,
                                            9.05137e-09,
                                        ],
                                        [
                                            2.56376e-09,
                                            5.08204e-09,
                                            1.16250e-08,
                                            1.24932e-08,
                                            6.21694e-09,
                                        ],
                                        [
                                            1.92028e-09,
                                            2.91597e-09,
                                            4.05485e-09,
                                            4.45941e-09,
                                            3.26985e-09,
                                        ],
                                    ],
                                    [
                                        [
                                            1.54650e-09,
                                            2.12664e-09,
                                            2.52480e-09,
                                            2.63839e-09,
                                            2.27090e-09,
                                        ],
                                        [
                                            1.96270e-09,
                                            3.01635e-09,
                                            4.06420e-09,
                                            4.52297e-09,
                                            3.22484e-09,
                                        ],
                                        [
                                            2.07894e-09,
                                            3.55250e-09,
                                            5.19654e-09,
                                            5.53920e-09,
                                            3.80806e-09,
                                        ],
                                        [
                                            1.88221e-09,
                                            3.01744e-09,
                                            4.25789e-09,
                                            4.39221e-09,
                                            3.14859e-09,
                                        ],
                                        [
                                            1.48635e-09,
                                            2.12507e-09,
                                            2.71580e-09,
                                            2.67292e-09,
                                            2.19256e-09,
                                        ],
                                    ],
                                ],
                                [
                                    [
                                        [
                                            1.03910e-09,
                                            1.38760e-09,
                                            1.70408e-09,
                                            1.79362e-09,
                                            1.47506e-09,
                                        ],
                                        [
                                            1.18030e-09,
                                            1.90408e-09,
                                            2.63724e-09,
                                            2.93031e-09,
                                            2.13124e-09,
                                        ],
                                        [
                                            1.33986e-09,
                                            2.22563e-09,
                                            3.39394e-09,
                                            3.74266e-09,
                                            2.51308e-09,
                                        ],
                                        [
                                            1.26758e-09,
                                            1.87892e-09,
                                            2.75481e-09,
                                            2.94163e-09,
                                            2.10264e-09,
                                        ],
                                        [
                                            1.00825e-09,
                                            1.37378e-09,
                                            1.65092e-09,
                                            1.87637e-09,
                                            1.51474e-09,
                                        ],
                                    ],
                                    [
                                        [
                                            1.22311e-09,
                                            1.93482e-09,
                                            2.67018e-09,
                                            2.86802e-09,
                                            2.14923e-09,
                                        ],
                                        [
                                            1.71324e-09,
                                            3.33310e-09,
                                            7.53755e-09,
                                            8.13377e-09,
                                            4.08728e-09,
                                        ],
                                        [
                                            1.87831e-09,
                                            4.26451e-09,
                                            1.55875e-08,
                                            1.64814e-08,
                                            6.00841e-09,
                                        ],
                                        [
                                            1.70510e-09,
                                            3.36658e-09,
                                            7.57682e-09,
                                            8.19842e-09,
                                            4.11423e-09,
                                        ],
                                        [
                                            1.22311e-09,
                                            2.01305e-09,
                                            2.73908e-09,
                                            2.84692e-09,
                                            2.10344e-09,
                                        ],
                                    ],
                                    [
                                        [
                                            1.35848e-09,
                                            2.30826e-09,
                                            3.37895e-09,
                                            3.58491e-09,
                                            2.63087e-09,
                                        ],
                                        [
                                            1.89266e-09,
                                            4.23544e-09,
                                            1.54325e-08,
                                            1.64309e-08,
                                            6.13497e-09,
                                        ],
                                        [
                                            2.19883e-09,
                                            5.88945e-09,
                                            3.12601e-08,
                                            3.28572e-08,
                                            9.48992e-09,
                                        ],
                                        [
                                            1.90293e-09,
                                            4.30858e-09,
                                            1.56431e-08,
                                            1.64648e-08,
                                            6.14781e-09,
                                        ],
                                        [
                                            1.36641e-09,
                                            2.30133e-09,
                                            3.44107e-09,
                                            3.64416e-09,
                                            2.68337e-09,
                                        ],
                                    ],
                                    [
                                        [
                                            1.22441e-09,
                                            1.93834e-09,
                                            2.69698e-09,
                                            2.99266e-09,
                                            2.17635e-09,
                                        ],
                                        [
                                            1.69640e-09,
                                            3.42237e-09,
                                            7.67935e-09,
                                            8.28690e-09,
                                            4.07757e-09,
                                        ],
                                        [
                                            1.95172e-09,
                                            4.38605e-09,
                                            1.58552e-08,
                                            1.64683e-08,
                                            5.96511e-09,
                                        ],
                                        [
                                            1.68959e-09,
                                            3.34921e-09,
                                            7.66122e-09,
                                            8.23337e-09,
                                            4.09714e-09,
                                        ],
                                        [
                                            1.26552e-09,
                                            1.92171e-09,
                                            2.67226e-09,
                                            2.93888e-09,
                                            2.15492e-09,
                                        ],
                                    ],
                                    [
                                        [
                                            1.01919e-09,
                                            1.40152e-09,
                                            1.66392e-09,
                                            1.73877e-09,
                                            1.49659e-09,
                                        ],
                                        [
                                            1.29348e-09,
                                            1.98786e-09,
                                            2.67842e-09,
                                            2.98077e-09,
                                            2.12526e-09,
                                        ],
                                        [
                                            1.37008e-09,
                                            2.34120e-09,
                                            3.42467e-09,
                                            3.65049e-09,
                                            2.50962e-09,
                                        ],
                                        [
                                            1.24043e-09,
                                            1.98858e-09,
                                            2.80607e-09,
                                            2.89459e-09,
                                            2.07501e-09,
                                        ],
                                        [
                                            9.79551e-10,
                                            1.40048e-09,
                                            1.78979e-09,
                                            1.76153e-09,
                                            1.44496e-09,
                                        ],
                                    ],
                                ],
                                [
                                    [
                                        [
                                            3.41644e-10,
                                            4.56226e-10,
                                            5.60282e-10,
                                            5.89722e-10,
                                            4.84982e-10,
                                        ],
                                        [
                                            3.88068e-10,
                                            6.26040e-10,
                                            8.67093e-10,
                                            9.63451e-10,
                                            7.00727e-10,
                                        ],
                                        [
                                            4.40529e-10,
                                            7.31762e-10,
                                            1.11589e-09,
                                            1.23054e-09,
                                            8.26272e-10,
                                        ],
                                        [
                                            4.16765e-10,
                                            6.17767e-10,
                                            9.05751e-10,
                                            9.67174e-10,
                                            6.91322e-10,
                                        ],
                                        [
                                            3.31500e-10,
                                            4.51684e-10,
                                            5.42804e-10,
                                            6.16929e-10,
                                            4.98029e-10,
                                        ],
                                    ],
                                    [
                                        [
                                            4.02144e-10,
                                            6.36145e-10,
                                            8.77922e-10,
                                            9.42972e-10,
                                            7.06641e-10,
                                        ],
                                        [
                                            5.63293e-10,
                                            1.09588e-09,
                                            2.47826e-09,
                                            2.67429e-09,
                                            1.34385e-09,
                                        ],
                                        [
                                            6.17565e-10,
                                            1.40212e-09,
                                            5.12499e-09,
                                            5.41889e-09,
                                            1.97549e-09,
                                        ],
                                        [
                                            5.60618e-10,
                                            1.10689e-09,
                                            2.49117e-09,
                                            2.69554e-09,
                                            1.35271e-09,
                                        ],
                                        [
                                            4.02145e-10,
                                            6.61867e-10,
                                            9.00578e-10,
                                            9.36034e-10,
                                            6.91588e-10,
                                        ],
                                    ],
                                    [
                                        [
                                            4.46653e-10,
                                            7.58928e-10,
                                            1.11096e-09,
                                            1.17868e-09,
                                            8.64999e-10,
                                        ],
                                        [
                                            6.22284e-10,
                                            1.39256e-09,
                                            5.07403e-09,
                                            5.40229e-09,
                                            2.01711e-09,
                                        ],
                                        [
                                            7.22948e-10,
                                            1.93638e-09,
                                            1.02780e-08,
                                            1.08031e-08,
                                            3.12017e-09,
                                        ],
                                        [
                                            6.25660e-10,
                                            1.41661e-09,
                                            5.14326e-09,
                                            5.41342e-09,
                                            2.02133e-09,
                                        ],
                                        [
                                            4.49260e-10,
                                            7.56649e-10,
                                            1.13138e-09,
                                            1.19816e-09,
                                            8.82259e-10,
                                        ],
                                    ],
                                    [
                                        [
                                            4.02572e-10,
                                            6.37303e-10,
                                            8.86734e-10,
                                            9.83953e-10,
                                            7.15558e-10,
                                        ],
                                        [
                                            5.57756e-10,
                                            1.12524e-09,
                                            2.52488e-09,
                                            2.72464e-09,
                                            1.34066e-09,
                                        ],
                                        [
                                            6.41702e-10,
                                            1.44208e-09,
                                            5.21302e-09,
                                            5.41458e-09,
                                            1.96126e-09,
                                        ],
                                        [
                                            5.55517e-10,
                                            1.10118e-09,
                                            2.51892e-09,
                                            2.70704e-09,
                                            1.34709e-09,
                                        ],
                                        [
                                            4.16087e-10,
                                            6.31836e-10,
                                            8.78609e-10,
                                            9.66270e-10,
                                            7.08513e-10,
                                        ],
                                    ],
                                    [
                                        [
                                            3.35096e-10,
                                            4.60803e-10,
                                            5.47077e-10,
                                            5.71689e-10,
                                            4.92060e-10,
                                        ],
                                        [
                                            4.25281e-10,
                                            6.53587e-10,
                                            8.80634e-10,
                                            9.80042e-10,
                                            6.98762e-10,
                                        ],
                                        [
                                            4.50466e-10,
                                            7.69759e-10,
                                            1.12599e-09,
                                            1.20024e-09,
                                            8.25133e-10,
                                        ],
                                        [
                                            4.07840e-10,
                                            6.53823e-10,
                                            9.22604e-10,
                                            9.51708e-10,
                                            6.82239e-10,
                                        ],
                                        [
                                            3.22065e-10,
                                            4.60462e-10,
                                            5.88462e-10,
                                            5.79171e-10,
                                            4.75086e-10,
                                        ],
                                    ],
                                ],
                            ]
                        ),
                        "error": np.array(
                            [
                                [
                                    [
                                        [0.02769, 0.02344, 0.02086, 0.02066, 0.02252],
                                        [0.02455, 0.01982, 0.01656, 0.01622, 0.01887],
                                        [0.02392, 0.01822, 0.01467, 0.01433, 0.01737],
                                        [0.02488, 0.01984, 0.01652, 0.01600, 0.01894],
                                        [0.02795, 0.02368, 0.02132, 0.02059, 0.02244],
                                    ],
                                    [
                                        [0.02496, 0.01987, 0.01686, 0.01638, 0.01902],
                                        [0.02155, 0.01520, 0.00979, 0.00969, 0.01351],
                                        [0.02027, 0.01338, 0.00677, 0.00697, 0.01117],
                                        [0.02127, 0.01524, 0.00974, 0.00973, 0.01349],
                                        [0.02470, 0.01980, 0.01666, 0.01634, 0.01877],
                                    ],
                                    [
                                        [0.02383, 0.01835, 0.01471, 0.01436, 0.01720],
                                        [0.02020, 0.01335, 0.00679, 0.00697, 0.01124],
                                        [0.01885, 0.01144, 0.00505, 0.00525, 0.00902],
                                        [0.02000, 0.01323, 0.00680, 0.00695, 0.01115],
                                        [0.02396, 0.01851, 0.01466, 0.01437, 0.01717],
                                    ],
                                    [
                                        [0.02500, 0.01971, 0.01656, 0.01619, 0.01885],
                                        [0.02141, 0.01526, 0.00976, 0.00965, 0.01355],
                                        [0.02036, 0.01323, 0.00678, 0.00699, 0.01120],
                                        [0.02127, 0.01504, 0.00977, 0.00970, 0.01358],
                                        [0.02482, 0.01975, 0.01660, 0.01626, 0.01886],
                                    ],
                                    [
                                        [0.02746, 0.02338, 0.02101, 0.02057, 0.02267],
                                        [0.02456, 0.01987, 0.01668, 0.01618, 0.01873],
                                        [0.02379, 0.01838, 0.01470, 0.01432, 0.01731],
                                        [0.02469, 0.01957, 0.01661, 0.01615, 0.01914],
                                        [0.02757, 0.02335, 0.02111, 0.02090, 0.02224],
                                    ],
                                ],
                                [
                                    [
                                        [0.02769, 0.02344, 0.02086, 0.02066, 0.02252],
                                        [0.02455, 0.01982, 0.01656, 0.01622, 0.01887],
                                        [0.02392, 0.01822, 0.01467, 0.01433, 0.01737],
                                        [0.02488, 0.01984, 0.01652, 0.01600, 0.01894],
                                        [0.02795, 0.02368, 0.02132, 0.02059, 0.02244],
                                    ],
                                    [
                                        [0.02496, 0.01987, 0.01686, 0.01638, 0.01902],
                                        [0.02155, 0.01520, 0.00979, 0.00969, 0.01351],
                                        [0.02027, 0.01338, 0.00677, 0.00697, 0.01117],
                                        [0.02127, 0.01524, 0.00974, 0.00973, 0.01349],
                                        [0.02470, 0.01980, 0.01666, 0.01634, 0.01877],
                                    ],
                                    [
                                        [0.02383, 0.01835, 0.01471, 0.01436, 0.01720],
                                        [0.02020, 0.01335, 0.00679, 0.00697, 0.01124],
                                        [0.01885, 0.01144, 0.00505, 0.00525, 0.00902],
                                        [0.02000, 0.01323, 0.00680, 0.00695, 0.01115],
                                        [0.02396, 0.01851, 0.01466, 0.01437, 0.01717],
                                    ],
                                    [
                                        [0.02500, 0.01971, 0.01656, 0.01619, 0.01885],
                                        [0.02141, 0.01526, 0.00976, 0.00965, 0.01355],
                                        [0.02036, 0.01323, 0.00678, 0.00699, 0.01120],
                                        [0.02127, 0.01504, 0.00977, 0.00970, 0.01358],
                                        [0.02482, 0.01975, 0.01660, 0.01626, 0.01886],
                                    ],
                                    [
                                        [0.02746, 0.02338, 0.02101, 0.02057, 0.02267],
                                        [0.02456, 0.01987, 0.01668, 0.01618, 0.01873],
                                        [0.02379, 0.01838, 0.01470, 0.01432, 0.01731],
                                        [0.02469, 0.01957, 0.01661, 0.01615, 0.01914],
                                        [0.02757, 0.02335, 0.02111, 0.02090, 0.02224],
                                    ],
                                ],
                                [
                                    [
                                        [0.02769, 0.02344, 0.02086, 0.02066, 0.02252],
                                        [0.02455, 0.01982, 0.01656, 0.01622, 0.01887],
                                        [0.02392, 0.01822, 0.01467, 0.01433, 0.01737],
                                        [0.02488, 0.01984, 0.01652, 0.01600, 0.01894],
                                        [0.02795, 0.02368, 0.02132, 0.02059, 0.02244],
                                    ],
                                    [
                                        [0.02496, 0.01987, 0.01686, 0.01638, 0.01902],
                                        [0.02155, 0.01520, 0.00979, 0.00969, 0.01351],
                                        [0.02027, 0.01338, 0.00677, 0.00697, 0.01117],
                                        [0.02127, 0.01524, 0.00974, 0.00973, 0.01349],
                                        [0.02470, 0.01980, 0.01666, 0.01634, 0.01877],
                                    ],
                                    [
                                        [0.02383, 0.01835, 0.01471, 0.01436, 0.01720],
                                        [0.02020, 0.01335, 0.00679, 0.00697, 0.01124],
                                        [0.01885, 0.01144, 0.00505, 0.00525, 0.00902],
                                        [0.02000, 0.01323, 0.00680, 0.00695, 0.01115],
                                        [0.02396, 0.01851, 0.01466, 0.01437, 0.01717],
                                    ],
                                    [
                                        [0.02500, 0.01971, 0.01656, 0.01619, 0.01885],
                                        [0.02141, 0.01526, 0.00976, 0.00965, 0.01355],
                                        [0.02036, 0.01323, 0.00678, 0.00699, 0.01120],
                                        [0.02127, 0.01504, 0.00977, 0.00970, 0.01358],
                                        [0.02482, 0.01975, 0.01660, 0.01626, 0.01886],
                                    ],
                                    [
                                        [0.02746, 0.02338, 0.02101, 0.02057, 0.02267],
                                        [0.02456, 0.01987, 0.01668, 0.01618, 0.01873],
                                        [0.02379, 0.01838, 0.01470, 0.01432, 0.01731],
                                        [0.02469, 0.01957, 0.01661, 0.01615, 0.01914],
                                        [0.02757, 0.02335, 0.02111, 0.02090, 0.02224],
                                    ],
                                ],
                                [
                                    [
                                        [0.02769, 0.02344, 0.02086, 0.02066, 0.02252],
                                        [0.02455, 0.01982, 0.01656, 0.01622, 0.01887],
                                        [0.02392, 0.01822, 0.01467, 0.01433, 0.01737],
                                        [0.02488, 0.01984, 0.01652, 0.01600, 0.01894],
                                        [0.02795, 0.02368, 0.02132, 0.02059, 0.02244],
                                    ],
                                    [
                                        [0.02496, 0.01987, 0.01686, 0.01638, 0.01902],
                                        [0.02155, 0.01520, 0.00979, 0.00969, 0.01351],
                                        [0.02027, 0.01338, 0.00677, 0.00697, 0.01117],
                                        [0.02127, 0.01524, 0.00974, 0.00973, 0.01349],
                                        [0.02470, 0.01980, 0.01666, 0.01634, 0.01877],
                                    ],
                                    [
                                        [0.02383, 0.01835, 0.01471, 0.01436, 0.01720],
                                        [0.02020, 0.01335, 0.00679, 0.00697, 0.01124],
                                        [0.01885, 0.01144, 0.00505, 0.00525, 0.00902],
                                        [0.02000, 0.01323, 0.00680, 0.00695, 0.01115],
                                        [0.02396, 0.01851, 0.01466, 0.01437, 0.01717],
                                    ],
                                    [
                                        [0.02500, 0.01971, 0.01656, 0.01619, 0.01885],
                                        [0.02141, 0.01526, 0.00976, 0.00965, 0.01355],
                                        [0.02036, 0.01323, 0.00678, 0.00699, 0.01120],
                                        [0.02127, 0.01504, 0.00977, 0.00970, 0.01358],
                                        [0.02482, 0.01975, 0.01660, 0.01626, 0.01886],
                                    ],
                                    [
                                        [0.02746, 0.02338, 0.02101, 0.02057, 0.02267],
                                        [0.02456, 0.01987, 0.01668, 0.01618, 0.01873],
                                        [0.02379, 0.01838, 0.01470, 0.01432, 0.01731],
                                        [0.02469, 0.01957, 0.01661, 0.01615, 0.01914],
                                        [0.02757, 0.02335, 0.02111, 0.02090, 0.02224],
                                    ],
                                ],
                            ]
                        ),
                    },
                ],
            },
        ),
        (
            "parser_test_data/fmesh3.m",
            {
                "date": "05/24/1812:21:45",
                "histories": 10000000.00,
                "title": "fmesh test",
                "tallies": [
                    {
                        "name": 14,
                        "particle": "NEUTRON",
                        "geom": "XYZ",
                        "bins": {
                            "ENERGY": np.array([0.00e00, 1.00e36]),
                            "X": np.array([-3.00, -1.00, 1.00, 3.00]),
                            "Y": np.array([-3.00, -1.00, 1.00, 3.00, 5.00]),
                            "Z": np.array([-4.00, -1.60, 0.80, 3.20, 5.60, 8.00]),
                        },
                        "result": np.array(
                            [
                                [
                                    [
                                        [
                                            7.18813e-03,
                                            3.06754e-03,
                                            1.46764e-03,
                                            7.66660e-04,
                                            4.03778e-04,
                                        ],
                                        [
                                            1.06634e-02,
                                            3.67344e-03,
                                            1.64976e-03,
                                            8.36825e-04,
                                            4.28819e-04,
                                        ],
                                        [
                                            7.17257e-03,
                                            3.06150e-03,
                                            1.47022e-03,
                                            7.71489e-04,
                                            3.98268e-04,
                                        ],
                                        [
                                            3.47599e-03,
                                            1.93182e-03,
                                            1.04730e-03,
                                            5.73177e-04,
                                            3.11514e-04,
                                        ],
                                    ],
                                    [
                                        [
                                            1.06743e-02,
                                            3.67618e-03,
                                            1.64985e-03,
                                            8.36179e-04,
                                            4.36976e-04,
                                        ],
                                        [
                                            2.01953e-02,
                                            4.44305e-03,
                                            1.83404e-03,
                                            9.03003e-04,
                                            4.60324e-04,
                                        ],
                                        [
                                            1.06720e-02,
                                            3.66319e-03,
                                            1.63652e-03,
                                            8.30197e-04,
                                            4.30752e-04,
                                        ],
                                        [
                                            4.24826e-03,
                                            2.24115e-03,
                                            1.18622e-03,
                                            6.40838e-04,
                                            3.43042e-04,
                                        ],
                                    ],
                                    [
                                        [
                                            7.21022e-03,
                                            3.08980e-03,
                                            1.48035e-03,
                                            7.68615e-04,
                                            4.02710e-04,
                                        ],
                                        [
                                            1.06646e-02,
                                            3.66923e-03,
                                            1.63603e-03,
                                            8.31973e-04,
                                            4.26487e-04,
                                        ],
                                        [
                                            7.19676e-03,
                                            3.07813e-03,
                                            1.47782e-03,
                                            7.74412e-04,
                                            4.05770e-04,
                                        ],
                                        [
                                            3.47633e-03,
                                            1.93512e-03,
                                            1.05235e-03,
                                            5.78607e-04,
                                            3.11651e-04,
                                        ],
                                    ],
                                ]
                            ]
                        ),
                        "error": np.array(
                            [
                                [
                                    [
                                        [0.00169, 0.00264, 0.00386, 0.00532, 0.00722],
                                        [0.00137, 0.00243, 0.00366, 0.00515, 0.00705],
                                        [0.00169, 0.00264, 0.00384, 0.00532, 0.00727],
                                        [0.00233, 0.00319, 0.00437, 0.00593, 0.00800],
                                    ],
                                    [
                                        [0.00137, 0.00243, 0.00367, 0.00514, 0.00698],
                                        [0.00093, 0.00221, 0.00350, 0.00497, 0.00682],
                                        [0.00137, 0.00243, 0.00368, 0.00517, 0.00703],
                                        [0.00212, 0.00302, 0.00419, 0.00572, 0.00774],
                                    ],
                                    [
                                        [0.00169, 0.00263, 0.00383, 0.00534, 0.00725],
                                        [0.00137, 0.00243, 0.00368, 0.00516, 0.00705],
                                        [0.00169, 0.00264, 0.00384, 0.00531, 0.00720],
                                        [0.00233, 0.00319, 0.00436, 0.00589, 0.00800],
                                    ],
                                ]
                            ]
                        ),
                    },
                    {
                        "name": 24,
                        "particle": "PHOTON",
                        "geom": "CYL",
                        "origin": np.array([-3.00e00, -3.00e00, -4.00e00]),
                        "axis": np.array([0.000e00, 0.000e00, 1.000e00]),
                        "bins": {
                            "ENERGY": np.array([0.00e00, 6.67e00, 1.33e01, 2.00e01]),
                            "R": np.array([0.00, 1.00, 2.00, 3.00]),
                            "Z": np.array([0.00, 1.25, 2.50, 3.75, 5.00]),
                            "THETA": np.array([0.000, 0.200, 0.400, 0.600, 0.800, 1.000]),
                        },
                        "result": np.array(
                            [
                                [
                                    [
                                        [
                                            4.26676e-04,
                                            3.60929e-04,
                                            2.51881e-04,
                                            2.35336e-04,
                                            3.27815e-04,
                                        ],
                                        [
                                            4.17376e-04,
                                            3.74891e-04,
                                            2.65612e-04,
                                            2.47303e-04,
                                            3.41729e-04,
                                        ],
                                        [
                                            3.52530e-04,
                                            3.21725e-04,
                                            2.39415e-04,
                                            2.22406e-04,
                                            2.87633e-04,
                                        ],
                                        [
                                            2.91647e-04,
                                            2.64957e-04,
                                            2.03075e-04,
                                            1.88207e-04,
                                            2.53038e-04,
                                        ],
                                    ],
                                    [
                                        [
                                            5.72802e-04,
                                            4.02577e-04,
                                            1.62808e-04,
                                            1.47284e-04,
                                            3.22879e-04,
                                        ],
                                        [
                                            5.32281e-04,
                                            3.90955e-04,
                                            1.77012e-04,
                                            1.56642e-04,
                                            3.20700e-04,
                                        ],
                                        [
                                            4.34871e-04,
                                            3.36510e-04,
                                            1.64430e-04,
                                            1.47865e-04,
                                            2.81179e-04,
                                        ],
                                        [
                                            3.39817e-04,
                                            2.74729e-04,
                                            1.44850e-04,
                                            1.30022e-04,
                                            2.32783e-04,
                                        ],
                                    ],
                                    [
                                        [
                                            7.68761e-04,
                                            4.02569e-04,
                                            1.09868e-04,
                                            9.83327e-05,
                                            2.85524e-04,
                                        ],
                                        [
                                            6.74185e-04,
                                            3.99908e-04,
                                            1.23025e-04,
                                            1.08080e-04,
                                            2.97412e-04,
                                        ],
                                        [
                                            5.11027e-04,
                                            3.42010e-04,
                                            1.15448e-04,
                                            1.05403e-04,
                                            2.57364e-04,
                                        ],
                                        [
                                            3.89689e-04,
                                            2.67553e-04,
                                            1.05158e-04,
                                            9.46154e-05,
                                            2.10578e-04,
                                        ],
                                    ],
                                ],
                                [
                                    [
                                        [
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            1.24150e-07,
                                            5.58994e-09,
                                            1.24827e-07,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            5.57853e-09,
                                            2.64984e-08,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                    ],
                                    [
                                        [
                                            3.31041e-08,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            4.24567e-08,
                                            4.34742e-08,
                                            4.24567e-08,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            0.00000e00,
                                            1.51344e-08,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            9.82692e-09,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                    ],
                                    [
                                        [
                                            3.44266e-08,
                                            3.21212e-08,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            4.55038e-08,
                                            3.04180e-08,
                                            2.54702e-08,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            3.03939e-09,
                                            3.13872e-08,
                                            4.05251e-09,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            2.19894e-09,
                                            4.43644e-09,
                                            3.78085e-08,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                    ],
                                ],
                                [
                                    [
                                        [
                                            1.61383e-07,
                                            2.45465e-07,
                                            4.80361e-08,
                                            2.86873e-08,
                                            1.36531e-07,
                                        ],
                                        [
                                            0.00000e00,
                                            0.00000e00,
                                            6.11344e-08,
                                            6.62558e-08,
                                            6.94414e-08,
                                        ],
                                        [
                                            1.79183e-07,
                                            1.83093e-07,
                                            1.26324e-07,
                                            1.10180e-07,
                                            9.84652e-08,
                                        ],
                                        [
                                            1.15801e-07,
                                            7.11787e-08,
                                            9.61169e-08,
                                            6.27373e-08,
                                            9.09577e-08,
                                        ],
                                    ],
                                    [
                                        [
                                            3.33988e-07,
                                            3.02723e-07,
                                            1.00276e-07,
                                            4.54345e-08,
                                            1.28529e-07,
                                        ],
                                        [
                                            1.05911e-07,
                                            0.00000e00,
                                            6.03481e-08,
                                            6.20807e-08,
                                            6.65379e-08,
                                        ],
                                        [
                                            1.03901e-07,
                                            1.16357e-07,
                                            1.77282e-07,
                                            6.12241e-08,
                                            1.68246e-07,
                                        ],
                                        [
                                            6.42099e-08,
                                            5.82408e-08,
                                            3.50897e-08,
                                            1.41587e-07,
                                            6.02894e-08,
                                        ],
                                    ],
                                    [
                                        [
                                            4.35790e-07,
                                            5.28413e-08,
                                            6.09709e-08,
                                            2.61146e-08,
                                            2.55932e-07,
                                        ],
                                        [
                                            1.36592e-07,
                                            2.03173e-07,
                                            6.40422e-08,
                                            2.55531e-08,
                                            1.59063e-07,
                                        ],
                                        [
                                            2.33628e-07,
                                            3.94338e-07,
                                            1.14898e-07,
                                            1.76035e-08,
                                            9.71856e-08,
                                        ],
                                        [
                                            2.10624e-07,
                                            1.56979e-07,
                                            7.03514e-08,
                                            4.10227e-08,
                                            1.43260e-07,
                                        ],
                                    ],
                                ],
                            ]
                        ),
                        "error": np.array(
                            [
                                [
                                    [
                                        [
                                            1.51165e-02,
                                            1.65362e-02,
                                            1.95638e-02,
                                            2.00471e-02,
                                            1.74294e-02,
                                        ],
                                        [
                                            1.54432e-02,
                                            1.65796e-02,
                                            1.93011e-02,
                                            1.98625e-02,
                                            1.73169e-02,
                                        ],
                                        [
                                            1.70852e-02,
                                            1.80477e-02,
                                            2.04493e-02,
                                            2.12808e-02,
                                            1.90279e-02,
                                        ],
                                        [
                                            1.91640e-02,
                                            2.00755e-02,
                                            2.24192e-02,
                                            2.31940e-02,
                                            2.06925e-02,
                                        ],
                                    ],
                                    [
                                        [
                                            9.09560e-03,
                                            1.13354e-02,
                                            1.68719e-02,
                                            1.73306e-02,
                                            1.27127e-02,
                                        ],
                                        [
                                            9.56337e-03,
                                            1.15957e-02,
                                            1.62495e-02,
                                            1.68587e-02,
                                            1.28723e-02,
                                        ],
                                        [
                                            1.07933e-02,
                                            1.25307e-02,
                                            1.69082e-02,
                                            1.73571e-02,
                                            1.37630e-02,
                                        ],
                                        [
                                            1.21292e-02,
                                            1.38951e-02,
                                            1.80609e-02,
                                            1.84843e-02,
                                            1.50679e-02,
                                        ],
                                    ],
                                    [
                                        [
                                            6.46985e-03,
                                            9.39907e-03,
                                            1.63465e-02,
                                            1.65062e-02,
                                            1.11913e-02,
                                        ],
                                        [
                                            7.10451e-03,
                                            9.57794e-03,
                                            1.55160e-02,
                                            1.59039e-02,
                                            1.11509e-02,
                                        ],
                                        [
                                            8.19050e-03,
                                            1.02979e-02,
                                            1.59845e-02,
                                            1.59556e-02,
                                            1.17764e-02,
                                        ],
                                        [
                                            9.40942e-03,
                                            1.15396e-02,
                                            1.66056e-02,
                                            1.68655e-02,
                                            1.28912e-02,
                                        ],
                                    ],
                                ],
                                [
                                    [
                                        [
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            1.00000e00,
                                            1.00000e00,
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            1.00000e00,
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                    ],
                                    [
                                        [
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            1.00000e00,
                                            1.00000e00,
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            0.00000e00,
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                    ],
                                    [
                                        [
                                            1.00000e00,
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            7.94765e-01,
                                            1.00000e00,
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            1.00000e00,
                                            1.00000e00,
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            1.00000e00,
                                            1.00000e00,
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                    ],
                                ],
                                [
                                    [
                                        [
                                            7.13649e-01,
                                            7.08199e-01,
                                            7.92007e-01,
                                            1.00000e00,
                                            7.98367e-01,
                                        ],
                                        [
                                            0.00000e00,
                                            0.00000e00,
                                            1.00000e00,
                                            1.00000e00,
                                            1.00000e00,
                                        ],
                                        [
                                            7.37976e-01,
                                            7.34192e-01,
                                            8.08038e-01,
                                            1.00000e00,
                                            9.90367e-01,
                                        ],
                                        [
                                            1.00000e00,
                                            1.00000e00,
                                            1.00000e00,
                                            7.11428e-01,
                                            1.00000e00,
                                        ],
                                    ],
                                    [
                                        [
                                            3.73939e-01,
                                            4.35674e-01,
                                            6.39167e-01,
                                            1.00000e00,
                                            6.13512e-01,
                                        ],
                                        [
                                            7.09827e-01,
                                            0.00000e00,
                                            8.65521e-01,
                                            8.21311e-01,
                                            7.07324e-01,
                                        ],
                                        [
                                            7.06117e-01,
                                            6.58996e-01,
                                            5.87822e-01,
                                            8.92465e-01,
                                            5.85102e-01,
                                        ],
                                        [
                                            5.99586e-01,
                                            7.93568e-01,
                                            1.00000e00,
                                            5.39828e-01,
                                            1.00000e00,
                                        ],
                                    ],
                                    [
                                        [
                                            2.74749e-01,
                                            6.07463e-01,
                                            7.11666e-01,
                                            1.00000e00,
                                            3.38896e-01,
                                        ],
                                        [
                                            4.70422e-01,
                                            4.07795e-01,
                                            7.12197e-01,
                                            1.00000e00,
                                            4.89133e-01,
                                        ],
                                        [
                                            4.69333e-01,
                                            3.23265e-01,
                                            4.92014e-01,
                                            8.57334e-01,
                                            5.38949e-01,
                                        ],
                                        [
                                            3.71867e-01,
                                            4.36175e-01,
                                            7.21109e-01,
                                            7.29208e-01,
                                            4.55335e-01,
                                        ],
                                    ],
                                ],
                            ]
                        ),
                    },
                ],
            },
        ),
        (
            "parser_test_data/fmesh2.m",
            {
                "date": "05/24/1812:21:45",
                "histories": 10000000.00,
                "title": "fmesh test",
                "tallies": [
                    {
                        "name": 24,
                        "particle": "PHOTON",
                        "geom": "CYL",
                        "origin": np.array([-3.00e00, -3.00e00, -4.00e00]),
                        "axis": np.array([0.000e00, 0.000e00, 1.000e00]),
                        "bins": {
                            "ENERGY": np.array([0.00e00, 6.67e00, 1.33e01, 2.00e01]),
                            "R": np.array([0.00, 1.00, 2.00, 3.00]),
                            "Z": np.array([0.00, 1.25, 2.50, 3.75, 5.00]),
                            "THETA": np.array([0.000, 0.200, 0.400, 0.600, 0.800, 1.000]),
                        },
                        "result": np.array(
                            [
                                [
                                    [
                                        [
                                            4.26676e-04,
                                            3.60929e-04,
                                            2.51881e-04,
                                            2.35336e-04,
                                            3.27815e-04,
                                        ],
                                        [
                                            4.17376e-04,
                                            3.74891e-04,
                                            2.65612e-04,
                                            2.47303e-04,
                                            3.41729e-04,
                                        ],
                                        [
                                            3.52530e-04,
                                            3.21725e-04,
                                            2.39415e-04,
                                            2.22406e-04,
                                            2.87633e-04,
                                        ],
                                        [
                                            2.91647e-04,
                                            2.64957e-04,
                                            2.03075e-04,
                                            1.88207e-04,
                                            2.53038e-04,
                                        ],
                                    ],
                                    [
                                        [
                                            5.72802e-04,
                                            4.02577e-04,
                                            1.62808e-04,
                                            1.47284e-04,
                                            3.22879e-04,
                                        ],
                                        [
                                            5.32281e-04,
                                            3.90955e-04,
                                            1.77012e-04,
                                            1.56642e-04,
                                            3.20700e-04,
                                        ],
                                        [
                                            4.34871e-04,
                                            3.36510e-04,
                                            1.64430e-04,
                                            1.47865e-04,
                                            2.81179e-04,
                                        ],
                                        [
                                            3.39817e-04,
                                            2.74729e-04,
                                            1.44850e-04,
                                            1.30022e-04,
                                            2.32783e-04,
                                        ],
                                    ],
                                    [
                                        [
                                            7.68761e-04,
                                            4.02569e-04,
                                            1.09868e-04,
                                            9.83327e-05,
                                            2.85524e-04,
                                        ],
                                        [
                                            6.74185e-04,
                                            3.99908e-04,
                                            1.23025e-04,
                                            1.08080e-04,
                                            2.97412e-04,
                                        ],
                                        [
                                            5.11027e-04,
                                            3.42010e-04,
                                            1.15448e-04,
                                            1.05403e-04,
                                            2.57364e-04,
                                        ],
                                        [
                                            3.89689e-04,
                                            2.67553e-04,
                                            1.05158e-04,
                                            9.46154e-05,
                                            2.10578e-04,
                                        ],
                                    ],
                                ],
                                [
                                    [
                                        [
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            1.24150e-07,
                                            5.58994e-09,
                                            1.24827e-07,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            5.57853e-09,
                                            2.64984e-08,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                    ],
                                    [
                                        [
                                            3.31041e-08,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            4.24567e-08,
                                            4.34742e-08,
                                            4.24567e-08,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            0.00000e00,
                                            1.51344e-08,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            9.82692e-09,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                    ],
                                    [
                                        [
                                            3.44266e-08,
                                            3.21212e-08,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            4.55038e-08,
                                            3.04180e-08,
                                            2.54702e-08,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            3.03939e-09,
                                            3.13872e-08,
                                            4.05251e-09,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            2.19894e-09,
                                            4.43644e-09,
                                            3.78085e-08,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                    ],
                                ],
                                [
                                    [
                                        [
                                            1.61383e-07,
                                            2.45465e-07,
                                            4.80361e-08,
                                            2.86873e-08,
                                            1.36531e-07,
                                        ],
                                        [
                                            0.00000e00,
                                            0.00000e00,
                                            6.11344e-08,
                                            6.62558e-08,
                                            6.94414e-08,
                                        ],
                                        [
                                            1.79183e-07,
                                            1.83093e-07,
                                            1.26324e-07,
                                            1.10180e-07,
                                            9.84652e-08,
                                        ],
                                        [
                                            1.15801e-07,
                                            7.11787e-08,
                                            9.61169e-08,
                                            6.27373e-08,
                                            9.09577e-08,
                                        ],
                                    ],
                                    [
                                        [
                                            3.33988e-07,
                                            3.02723e-07,
                                            1.00276e-07,
                                            4.54345e-08,
                                            1.28529e-07,
                                        ],
                                        [
                                            1.05911e-07,
                                            0.00000e00,
                                            6.03481e-08,
                                            6.20807e-08,
                                            6.65379e-08,
                                        ],
                                        [
                                            1.03901e-07,
                                            1.16357e-07,
                                            1.77282e-07,
                                            6.12241e-08,
                                            1.68246e-07,
                                        ],
                                        [
                                            6.42099e-08,
                                            5.82408e-08,
                                            3.50897e-08,
                                            1.41587e-07,
                                            6.02894e-08,
                                        ],
                                    ],
                                    [
                                        [
                                            4.35790e-07,
                                            5.28413e-08,
                                            6.09709e-08,
                                            2.61146e-08,
                                            2.55932e-07,
                                        ],
                                        [
                                            1.36592e-07,
                                            2.03173e-07,
                                            6.40422e-08,
                                            2.55531e-08,
                                            1.59063e-07,
                                        ],
                                        [
                                            2.33628e-07,
                                            3.94338e-07,
                                            1.14898e-07,
                                            1.76035e-08,
                                            9.71856e-08,
                                        ],
                                        [
                                            2.10624e-07,
                                            1.56979e-07,
                                            7.03514e-08,
                                            4.10227e-08,
                                            1.43260e-07,
                                        ],
                                    ],
                                ],
                            ]
                        ),
                        "error": np.array(
                            [
                                [
                                    [
                                        [
                                            1.51165e-02,
                                            1.65362e-02,
                                            1.95638e-02,
                                            2.00471e-02,
                                            1.74294e-02,
                                        ],
                                        [
                                            1.54432e-02,
                                            1.65796e-02,
                                            1.93011e-02,
                                            1.98625e-02,
                                            1.73169e-02,
                                        ],
                                        [
                                            1.70852e-02,
                                            1.80477e-02,
                                            2.04493e-02,
                                            2.12808e-02,
                                            1.90279e-02,
                                        ],
                                        [
                                            1.91640e-02,
                                            2.00755e-02,
                                            2.24192e-02,
                                            2.31940e-02,
                                            2.06925e-02,
                                        ],
                                    ],
                                    [
                                        [
                                            9.09560e-03,
                                            1.13354e-02,
                                            1.68719e-02,
                                            1.73306e-02,
                                            1.27127e-02,
                                        ],
                                        [
                                            9.56337e-03,
                                            1.15957e-02,
                                            1.62495e-02,
                                            1.68587e-02,
                                            1.28723e-02,
                                        ],
                                        [
                                            1.07933e-02,
                                            1.25307e-02,
                                            1.69082e-02,
                                            1.73571e-02,
                                            1.37630e-02,
                                        ],
                                        [
                                            1.21292e-02,
                                            1.38951e-02,
                                            1.80609e-02,
                                            1.84843e-02,
                                            1.50679e-02,
                                        ],
                                    ],
                                    [
                                        [
                                            6.46985e-03,
                                            9.39907e-03,
                                            1.63465e-02,
                                            1.65062e-02,
                                            1.11913e-02,
                                        ],
                                        [
                                            7.10451e-03,
                                            9.57794e-03,
                                            1.55160e-02,
                                            1.59039e-02,
                                            1.11509e-02,
                                        ],
                                        [
                                            8.19050e-03,
                                            1.02979e-02,
                                            1.59845e-02,
                                            1.59556e-02,
                                            1.17764e-02,
                                        ],
                                        [
                                            9.40942e-03,
                                            1.15396e-02,
                                            1.66056e-02,
                                            1.68655e-02,
                                            1.28912e-02,
                                        ],
                                    ],
                                ],
                                [
                                    [
                                        [
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            1.00000e00,
                                            1.00000e00,
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            1.00000e00,
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                    ],
                                    [
                                        [
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            1.00000e00,
                                            1.00000e00,
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            0.00000e00,
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                    ],
                                    [
                                        [
                                            1.00000e00,
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            7.94765e-01,
                                            1.00000e00,
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            1.00000e00,
                                            1.00000e00,
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                        [
                                            1.00000e00,
                                            1.00000e00,
                                            1.00000e00,
                                            0.00000e00,
                                            0.00000e00,
                                        ],
                                    ],
                                ],
                                [
                                    [
                                        [
                                            7.13649e-01,
                                            7.08199e-01,
                                            7.92007e-01,
                                            1.00000e00,
                                            7.98367e-01,
                                        ],
                                        [
                                            0.00000e00,
                                            0.00000e00,
                                            1.00000e00,
                                            1.00000e00,
                                            1.00000e00,
                                        ],
                                        [
                                            7.37976e-01,
                                            7.34192e-01,
                                            8.08038e-01,
                                            1.00000e00,
                                            9.90367e-01,
                                        ],
                                        [
                                            1.00000e00,
                                            1.00000e00,
                                            1.00000e00,
                                            7.11428e-01,
                                            1.00000e00,
                                        ],
                                    ],
                                    [
                                        [
                                            3.73939e-01,
                                            4.35674e-01,
                                            6.39167e-01,
                                            1.00000e00,
                                            6.13512e-01,
                                        ],
                                        [
                                            7.09827e-01,
                                            0.00000e00,
                                            8.65521e-01,
                                            8.21311e-01,
                                            7.07324e-01,
                                        ],
                                        [
                                            7.06117e-01,
                                            6.58996e-01,
                                            5.87822e-01,
                                            8.92465e-01,
                                            5.85102e-01,
                                        ],
                                        [
                                            5.99586e-01,
                                            7.93568e-01,
                                            1.00000e00,
                                            5.39828e-01,
                                            1.00000e00,
                                        ],
                                    ],
                                    [
                                        [
                                            2.74749e-01,
                                            6.07463e-01,
                                            7.11666e-01,
                                            1.00000e00,
                                            3.38896e-01,
                                        ],
                                        [
                                            4.70422e-01,
                                            4.07795e-01,
                                            7.12197e-01,
                                            1.00000e00,
                                            4.89133e-01,
                                        ],
                                        [
                                            4.69333e-01,
                                            3.23265e-01,
                                            4.92014e-01,
                                            8.57334e-01,
                                            5.38949e-01,
                                        ],
                                        [
                                            3.71867e-01,
                                            4.36175e-01,
                                            7.21109e-01,
                                            7.29208e-01,
                                            4.55335e-01,
                                        ],
                                    ],
                                ],
                            ]
                        ),
                    }
                ],
            },
        ),
    ],
)
def test_meshtal_parser(mesh_file, expected):
    mesh_file = file_resolver(mesh_file)
    with open(mesh_file) as f:
        text = f.read() + "\n"
    meshtal_lexer.begin("INITIAL")
    tallies = meshtal_parser.parse(text, lexer=meshtal_lexer)
    assert tallies.keys() == expected.keys()
    for k in ["date", "histories", "title"]:
        assert tallies[k] == expected[k]
    for t, a in zip(tallies["tallies"], expected["tallies"]):
        for k in ["name", "particle", "geom"]:
            assert t[k] == a[k]
        for k, v in a["bins"].items():
            np.testing.assert_array_almost_equal(t["bins"][k], v)
        np.testing.assert_array_almost_equal(t["result"], a["result"])
        np.testing.assert_array_almost_equal(t["error"], a["error"])


if __name__ == "__main__":
    pytest.main()
